# src\file_conversor\backend\gui\flask_api.py

import tempfile
import threading

from pathlib import Path
from typing import Any, Callable, Iterable, Self

from flask import Flask, Response, request, jsonify

# user-provided modules
from file_conversor.backend.gui.flask_api_status import FlaskApiStatus, FlaskApiStatusCompleted, FlaskApiStatusUnknown, FlaskApiStatusError, FlaskApiStatusProcessing, FlaskApiStatusReady

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

from file_conversor.utils.formatters import escape_xml, format_traceback_html, format_traceback_str, parse_js_to_py

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


class FlaskApi:
    _status: dict[str, FlaskApiStatus] = {}

    @classmethod
    def _set_state(cls, params: dict[str, Any]) -> None:
        """Set the application state based on the provided parameters."""
        STATE['quiet'] = bool(params.get('quiet', False))
        STATE['verbose'] = bool(params.get('verbose', False))
        STATE['overwrite-output'] = bool(params.get('overwrite-output', False))

    @classmethod
    def _execute_thread(
        cls,
        params: dict[str, Any],
        callback: Callable[[dict[str, Any], FlaskApiStatus], None],
    ) -> None:
        """Thread to handle file processing."""
        try:
            logger.debug(f"Processing thread started ...")
            cls._set_state(params)

            status = cls._get_status(params['status_id'])
            status.set_progress(-1)  # indeterminate progress

            # run callback
            callback(params, status)

            # finished successfully
            msg_success = _('Processing completed successfully.')
            status.set(FlaskApiStatusCompleted(
                id=status.get_id(),
                message=status.get_message() or msg_success,
            ))
            logger.info(f"[bold]{msg_success}[/]")
        except Exception as e:
            logger.error(format_traceback_str(e, debug=STATE["debug"]))
            progress = status.get_progress()
            status.set(FlaskApiStatusError(
                id=status.get_id(),
                exception=format_traceback_html(e, debug=STATE["debug"]),
                progress=None if progress is None or progress < 0 else progress,
            ))
            raise

    @classmethod
    def _add_status(cls) -> FlaskApiStatus:
        """Add or update the status of a specific operation by its ID."""
        status_id = len(cls._status) + 1
        status = FlaskApiStatusProcessing(id=status_id)
        cls._status[status._id] = status
        return status

    @classmethod
    def _get_status(cls, status_id: str | None) -> FlaskApiStatus:
        """Get the status of a specific operation by its ID."""
        if status_id is None or status_id.lower() in ('', 'none', 'null', '0'):
            return FlaskApiStatusReady()
        status = cls._status.get(status_id)
        # remove completed or error statuses
        if status is not None and isinstance(status, (FlaskApiStatusCompleted, FlaskApiStatusError)):
            del cls._status[status_id]
        return status or FlaskApiStatusUnknown(status_id)

    @classmethod
    def get_args(cls) -> dict[str, Any]:
        """
        Get query parameters from the request.

        :return: dict[str, Any]: A dictionary containing the query parameters.
        """
        data: dict[str, str] = request.args.to_dict()
        logger.debug(f"Received args: {data}")
        for key, value in data.items():
            data[key] = parse_js_to_py(value)
        return data

    @classmethod
    def get_form_data(cls) -> dict[str, Any]:
        """
        Get form data from the request.

        :return: dict[str, Any]: A dictionary containing the form data.
        """
        data: dict[str, str] = request.form.to_dict()
        logger.debug(f"Received data: {data}")
        # Update the configuration with the provided data
        for key, value in data.items():
            data[key] = parse_js_to_py(value)
        return data

    @classmethod
    def get_files_list(cls, key: str) -> tuple[list[Path], Path]:
        """
        Save the uploaded files from the request into temporary dir, for processing.

        :return: tuple[list[Path], Path]: A tuple containing the list of saved file paths and the temporary directory path.
        """
        res: list[Path] = []
        files = request.files.getlist(key)
        temp_dir = Path(tempfile.mkdtemp())
        for file in files:
            if not file.filename:
                continue
            file_path = temp_dir / file.filename
            file.save(file_path)
            res.append(file_path)
        return res, temp_dir

    @classmethod
    def status_response(cls, id: str | None) -> tuple[Response, int]:
        """
        API endpoint to get the application status.

        :param id: str | None: The ID of the status to retrieve.

        :return: tuple[str, int]: ({id:str, status:str, message:str, exception:str, progress:int}, status_code:int)
        """
        status = cls._get_status(id)
        ret_code = 200
        if isinstance(status, FlaskApiStatusUnknown):
            ret_code = 404
        elif isinstance(status, FlaskApiStatusError):
            ret_code = 500
        return jsonify(status.json()), ret_code

    @classmethod
    def execute_response(
        cls,
        callback: Callable[[dict[str, Any], FlaskApiStatus], None],
    ) -> tuple[Response, int]:
        """
        API endpoint to process files.
        ```python
        req={
            'param1': 'Value 1',
            'param2': 'Value 2',
            ...
        }
        ```
        :param callback: Callable[[dict[str, Any]], None]: The function to execute in a separate thread.

        :return: tuple[str, int]: ({status:str, status_id:str, message:str}, 200)
        """
        status = None
        try:
            logger.info(f"[bold]{_('File processing requested via API.')}[/]")
            data = cls.get_form_data()
            status = cls._add_status()
            data['status_id'] = status.get_id()
            threading.Thread(target=cls._execute_thread, args=(data, callback), daemon=True).start()
            return jsonify(status.json()), 200
        except Exception as e:
            logger.error(format_traceback_str(e, debug=STATE["debug"]))
            return jsonify(FlaskApiStatusError(
                id=status.get_id() if status else -1,
                exception=format_traceback_html(e, debug=STATE["debug"]),
            ).json()), 500


__all__ = ["FlaskApi"]




# src\file_conversor\cli\win\restart_exp_cmd.py

import typer

from typing import Annotated

from rich import print


# user-provided modules
from file_conversor.cli.win._typer import OTHERS_PANEL as RICH_HELP_PANEL
from file_conversor.cli.win._typer import COMMAND_NAME, RESTART_EXPLORER_NAME

from file_conversor.config import Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.system import win

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = set()


@typer_cmd.command(
    name=RESTART_EXPLORER_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Restarts explorer.exe (to refresh ctx menus).')}        
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor {COMMAND_NAME} {RESTART_EXPLORER_NAME}` 
""")
def restart_explorer():
    logger.info(f"{_('Restarting explorer.exe')} ...")
    win.restart_explorer()


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]




# src\file_conversor\cli\doc\convert_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend import LibreofficeWriterBackend

from file_conversor.cli.doc._typer import COMMAND_NAME, CONVERT_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.formatters import format_in_out_files_tuple
from file_conversor.utils.typer_utils import FormatOption, InputFilesArgument, OutputDirOption


from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

# typer PANELS
typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = LibreofficeWriterBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # WordBackend commands
    for ext in LibreofficeWriterBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name=f"to_{ext}",
                description=f"To {ext.upper()}",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{CONVERT_NAME}" "%1" -f "{ext}""',
                icon=str(icons_folder_path / f"{ext}.ico"),
            )
            for ext in LibreofficeWriterBackend.SUPPORTED_OUT_FORMATS
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_doc_convert_cmd(
    input_files: List[Path],
    format: str,
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
) -> None:
    files = format_in_out_files_tuple(
        input_files=input_files,
        output_dir=output_dir,
        format=format,
    )

    backend = LibreofficeWriterBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
    )

    with ProgressManager(len(input_files)) as progress_mgr:
        logger.info(f"[bold]{_('Converting files')}[/] ...")
        # Perform conversion
        backend.convert(
            files=files,
            file_processed_callback=lambda _: progress_callback(progress_mgr.complete_step())
        )

    logger.info(f"{_('File conversion')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


@typer_cmd.command(
    name=CONVERT_NAME,
    help=f"""
        {_('Convert document files into other formats (requires LibreOffice).')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.odt -f doc`

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.docx -f pdf`

        - `file_conversor {COMMAND_NAME} {CONVERT_NAME} input_file.pdf -f docx`
    """)
def convert(
    input_files: Annotated[List[Path], InputFilesArgument(LibreofficeWriterBackend)],
    format: Annotated[str, FormatOption(LibreofficeWriterBackend)],
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_doc_convert_cmd(
        input_files=input_files,
        format=format,
        output_dir=output_dir,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_doc_convert_cmd",
]




# src\file_conversor\config\__init__.py

"""
This module initializes the app configuration modules.
"""

from file_conversor.config.abstract_singleton_thread_safe import *
from file_conversor.config.cleanup import *
from file_conversor.config.config import *
from file_conversor.config.environment import *
from file_conversor.config.locale import *
from file_conversor.config.log import *
from file_conversor.config.state import *



# src\file_conversor\backend\ebook\calibre_backend.py

"""
This module provides functionalities for handling ebook files using Calibre.
"""

from pathlib import Path
from typing import Any, Callable, Iterable

# user-provided imports
from file_conversor.backend.abstract_backend import AbstractBackend
from file_conversor.config import Environment, Log, get_translation

from file_conversor.dependency import BrewPackageManager, ScoopPackageManager

from file_conversor.utils.validators import check_file_format

_ = get_translation()
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class CalibreBackend(AbstractBackend):
    """
    CalibreBackend is a class that provides an interface for handling ebook files using Calibre.
    """

    SUPPORTED_IN_FORMATS = {
        'azw': {},
        'azw3': {},
        'azw4': {},
        'cbr': {},
        'cbz': {},
        'epub': {},
        'fb2': {},
        'mobi': {},
    }

    SUPPORTED_OUT_FORMATS = {
        'azw3': {},
        'docx': {},
        'epub': {},
        'fb2': {},
        'mobi': {},
        'pdf': {},
    }

    EXTERNAL_DEPENDENCIES = {
        "ebook-convert",
    }

    def __init__(
        self,
        install_deps: bool | None,
        verbose: bool = False,
    ):
        """
        Initialize the Calibre backend.

        :param install_deps: Install external dependencies. If True auto install using a package manager. If False, do not install external dependencies. If None, asks user for action. 
        :param verbose: Verbose logging. Defaults to False.      

        :raises RuntimeError: if calibre dependency is not found
        """
        super().__init__(
            pkg_managers={
                ScoopPackageManager({
                    "ebook-convert": "calibre"
                }, buckets=[
                    "extras",
                ], env=[
                    rf"{Path.home()}\scoop\shims"
                ]),
                BrewPackageManager({
                    "ebook-convert": "calibre"
                }),
            },
            install_answer=install_deps,
        )
        self._install_deps = install_deps
        self._verbose = verbose

        # check ebook-convert
        self._ebook_convert_bin = self.find_in_path("ebook-convert")

    def convert(
        self,
        output_file: str | Path,
        input_file: str | Path,
    ):
        """
        Convert input file into an output file.

        :param output_file: Output file.
        :param input_file: Input file.        

        :raises FileNotFoundError: if input file not found.
        """
        self.check_file_exists(input_file)

        input_path = Path(input_file).resolve()
        output_path = Path(output_file).resolve()

        # Execute command
        process = Environment.run(
            str(self._ebook_convert_bin),
            str(input_path),
            str(output_path),
        )
        return process


__all__ = [
    'CalibreBackend',
]



# src/file_conversor/backend/gui/config/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui.config._index import config_index


def routes():
    return [
        FlaskRoute(
            rule="/config",
            handler=config_index
        ),
    ]


__all__ = [
    'config_index',

    'routes',
]



# src/file_conversor/backend/gui/image/to_pdf.py

from pathlib import Path
from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.image import Img2PDFBackend

from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.image._dom_page import image_index_nav_item, image_to_pdf_nav_item

from file_conversor.utils.bulma_utils import InputFilesField, OutputFileField, ImageDPIField, ImageFitField, ImagePageSizeField, ImageSetMetadataField
from file_conversor.utils.dominate_bulma import PageForm

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageImageToPDF():
    return PageForm(
        InputFilesField(
            *[f for f in Img2PDFBackend.SUPPORTED_IN_FORMATS],
            description=_("Image files"),
        ),
        ImageDPIField(),
        ImageFitField(),
        ImagePageSizeField(),
        OutputFileField(
            *[
                (f, f'{f.upper()} {_("file")}')
                for f in Img2PDFBackend.SUPPORTED_OUT_FORMATS
            ],
            current_value="output.pdf",
        ),
        ImageSetMetadataField(),
        api_endpoint=f"{url_for('api_image_to_pdf')}",
        nav_items=[
            home_nav_item(),
            image_index_nav_item(),
            image_to_pdf_nav_item(active=True),
        ],
        _title=f"{_('Image To PDF')} - File Conversor",
    )


def image_to_pdf():
    return render_template_string(str(
        PageImageToPDF()
    ))



# src\file_conversor\gui\web_app.py

import webview
import webbrowser
import typer
import time
import threading

from flask import Flask, request
from pathlib import Path
from typing import Any, Callable, Iterable, Self

# user-provided modules
from file_conversor.backend.gui._webview_api import WebViewAPI
from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.config import Configuration, Log, State, Environment, AbstractSingletonThreadSafe
from file_conversor.config.locale import AVAILABLE_LANGUAGES, get_system_locale, get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


# Create a web application
class WebApp(AbstractSingletonThreadSafe):
    LOCALIZATION = {
        'global.quitConfirmation': _('Do you really want to quit?'),
        'global.ok': _('OK'),
        'global.quit': _('Quit'),
        'global.cancel': _('Cancel'),
        'global.saveFile': _('Save file'),
        'cocoa.menu.about': _('About'),
        'cocoa.menu.services': _('Services'),
        'cocoa.menu.view': _('View'),
        'cocoa.menu.edit': _('Edit'),
        'cocoa.menu.hide': _('Hide'),
        'cocoa.menu.hideOthers': _('Hide Others'),
        'cocoa.menu.showAll': _('Show All'),
        'cocoa.menu.quit': _('Quit'),
        'cocoa.menu.fullscreen': _('Enter Fullscreen'),
        'cocoa.menu.cut': _('Cut'),
        'cocoa.menu.copy': _('Copy'),
        'cocoa.menu.paste': _('Paste'),
        'cocoa.menu.selectAll': _('Select All'),
        'windows.fileFilter.allFiles': _('All files'),
        'windows.fileFilter.otherFiles': _('Other file types'),
        'linux.openFile': _('Open file'),
        'linux.openFiles': _('Open files'),
        'linux.openFolder': _('Open folder'),
    }

    def __init__(self) -> None:
        super().__init__()
        web_path = Environment.get_web_folder()

        self.webview_api = WebViewAPI()
        window = webview.create_window(
            title="File Conversor",
            url=f"http://127.0.0.1:{CONFIG['port']}",
            localization=self.LOCALIZATION,
            maximized=True,
            js_api=self.webview_api,
        )
        if window is None:
            raise RuntimeError("Failed to create webview window.")
        self.window: webview.Window = window
        self.app = Flask(
            __name__,
            static_folder=web_path / 'static',
            template_folder=web_path / 'templates',
        )
        self._flask_ready = threading.Event()
        self.routes: set[FlaskRoute] = set()

    def _open_browser(self, url: str) -> None:
        # new=0 → same window (if possible)
        # new=1 → new window
        # new=2 → new tab (recommended)
        webbrowser.open(url, new=2)

    def _run_flask(self, testing: bool = False) -> Flask:
        logger.info(f"{_('Registering routes ...')}")
        for route in self.routes:
            route.register(self.app)
            # logger.debug(f"Route registered: {route.rule} -> {route.options}")
        if testing:
            logger.info(f"{_('Returning Flask object in testing mode ...')}")
            self.app.config.update({
                'TESTING': True,
                'SECRET_KEY': 'testing',
            })
            return self.app
        logger.info(f"{_('[bold] Starting Flask server ... [/]')}")
        while True:
            try:
                self._flask_ready.set()
                self.app.run(
                    host="127.0.0.1",
                    port=CONFIG["port"],
                    debug=STATE["debug"],
                    threaded=True,  # Allow handling multiple requests
                    use_reloader=False,   # Disable the reloader to avoid issues with threading
                )
            except (typer.Abort, typer.Exit, KeyboardInterrupt, SystemExit) as e:
                logger.info(f"{_('Attempting to shut down Flask server ...')}")
                break
            except Exception as e:
                logger.error(f"Error in Flask server: {repr(e)}")
                time.sleep(0.100)  # Wait before retrying
        logger.info(f"[bold]{_('Flask server stopped.')}[/]")
        return self.app

    def _run_webview(self) -> None:
        logger.info(f"[bold]{_('Starting webview window ...')}[/]")
        webview.start(
            self.__init_webview,
            debug=STATE['debug'],
            icon=str(Environment.get_app_icon()),
        )
        logger.info(f"[bold]{_('Webview window closed.')}[/]")

    def __init_webview(self) -> None:
        # Listen for reloads (new pages)
        self.window.events.loaded += self.webview_api._on_load  # pyright: ignore[reportOperatorIssue]

    def expose(self, *func: Callable[..., Any]) -> Self:
        """Expose a new function to the webview JS API."""
        self.window.expose(*func)
        return self

    def evaluate_js(
            self,
            script: str,
            callback: Callable[..., Any] | None = None,
    ) -> Any:
        """Evaluate a JavaScript script in the webview window."""
        return self.window.evaluate_js(script, callback)

    def add_route(self, routes: FlaskRoute | Iterable[FlaskRoute]) -> Self:
        """Add a new route to the Flask application."""
        for route in (routes if isinstance(routes, Iterable) else [routes]):
            if route in self.routes:
                raise RuntimeError(f"Route already exists: {route}")
            self.routes.add(route)
        return self

    def add_context_processor(self, func: Callable[..., dict[str, Any]]) -> Self:
        """Add a new context processor to the Flask application."""
        self.app.context_processor(func)
        return self

    def run(self) -> None:
        """Run the Flask server and the webview window."""
        flask_thread = threading.Thread(target=self._run_flask, daemon=True)
        flask_thread.start()

        logger.debug(f"{_('Waiting for Flask to be ready ...')}")
        self._flask_ready.wait()

        url = f"http://127.0.0.1:{CONFIG['port']}"
        try:
            logger.info(f"[bold]{_('Access the app at')} {url}[/]")
            logger.info(f"[bold]{_('Press Ctrl+C to exit.')}[/]")
            self._run_webview()
        except (typer.Abort, typer.Exit, KeyboardInterrupt, SystemExit):
            logger.info(f"{_('Shutting down main thread...')}")

        logger.info(f"{_('[bold] Shutting down the application ... [/]')}")
        raise typer.Exit(0)


__all__ = ["WebApp"]




# src/file_conversor/utils/__init__.py

"""
This module initializes the utils package.
It can contain utility functions or classes that are used across the application.
"""

from file_conversor.utils.abstract_register_manager import *
from file_conversor.utils.command_manager import *
from file_conversor.utils.progress_manager import *

from file_conversor.utils.rich_utils import *



# src\file_conversor\config\config.py

import json
import locale

from pathlib import Path
from typing import Any

from file_conversor.config.environment import Environment
from file_conversor.config.abstract_singleton_thread_safe import AbstractSingletonThreadSafe


class Configuration(AbstractSingletonThreadSafe):

    def __init__(self) -> None:
        super().__init__()

        self.__config_path = Environment.get_data_folder() / ".config.json"

        # Load configuration dictionary
        self.__data = {}
        self.load()

    def __repr__(self) -> str:
        return repr(self.__data)

    def __str__(self) -> str:
        return str(self.__data)

    def __getitem__(self, key) -> Any:
        return self.__data[key]

    def __setitem__(self, key, value):
        self.__data[key] = value

    def __delitem__(self, key):
        del self.__data[key]

    def __contains__(self, key):
        return key in self.__data

    def __len__(self):
        return len(self.__data)

    def get_path(self) -> Path:
        return self.__config_path

    def to_dict(self):
        return self.__data.copy()

    def update(self, new: dict):
        self.__data.update(new)

    def save(self):
        """Save app configuration file"""
        self.__config_path.write_text(json.dumps(self.__data, indent=2))

    def load(self):
        """Load app configuration file"""
        self.reset_factory_default()
        if self.__config_path.exists():
            self.__data.update(json.loads(self.__config_path.read_text()))

    def reset_factory_default(self):
        language = "en_US"
        if locale.getlocale() and locale.getlocale()[0]:
            language = locale.getlocale()[0]

        self.__data.clear()
        self.__data.update({
            "cache-enabled": True,     # Enable HTTP cache
            "cache-expire-after": int(30 * 24 * 60 * 60),  # HTTP cache expiration time in seconds (30 days)
            "port": 5000,              # Default port (flask app)
            "language": language,      # Default: system language or "en_US"
            "install-deps": True,      # Default: ask user to confirm dependency installation
            "audio-bitrate": 0,        # Default audio bitrate in kbps
            "video-bitrate": 0,        # Default video bitrate in kbps
            "video-format": "mp4",     # Default video format
            "video-encoding-speed": "medium",  # Default video encoding speed
            "video-quality": "medium",  # Default video quality
            "image-quality": 90,        # Default image quality 90%
            "image-dpi": 200,           # Default image => PDF dpi
            "image-fit": 'into',        # Default image => PDF fit mode
            "image-page-size": None,    # Default image => PDF page size
            "image-resampling": "bicubic",  # Default image resampling algorithm
            "pdf-compression": "medium",  # Default PDF compression level
            "gui-zoom": 100,            # Default GUI zoom level
            "gui-output-dir": str(Environment.UserFolder.DOWNLOADS()),  # Default output directory
        })


__all__ = [
    "Configuration",
]



# src\file_conversor\utils\bulma\form_file_list.py

from typing import Any, Sequence

from file_conversor.utils.dominate_utils import button, div, i, input_, label, option, select, template, p
from file_conversor.utils.formatters import format_py_to_js


def _SelectBox(
        size: int = 8,
        _multiple: bool = True,
):
    """
    Create the select box for the file list.

    Needs alpine data with:
    - files: list of available files
    - selected: list of selected files
    - isValid: boolean indicating if the selection is valid

    :param size: The number of visible options in the select box.
    :param _multiple: Whether to allow multiple selection.

    :return: The select box element.
    """
    # Select box
    with div(_class="control is-flex is-flex-direction-column is-flex-grow-1") as control_left:
        with div(
            _class="select is-multiple is-flex is-flex-grow-1",
            **{
                ":class": """{
                        'is-danger': !isValid,
                        'is-success': isValid,
                    }""",
            },
        ):
            with select(
                _multiple=_multiple,
                _size=size,
                _class="is-full-width",
                _style="cursor: unset;",
                **{
                    "x-model": "selected",
                    ":title": "help",
                },
            ):
                with template(**{"x-for": "(opt, index) in files", ":key": "opt"}):
                    option(
                        **{
                            ":value": "opt",
                            "x-text": "`#${index + 1} - ${opt}`",
                        }
                    )
        p(
            _class="help",
            **{
                ":class": """{
                    'is-danger': !isValid,
                    'is-hidden': isValid,
                }""",
                "x-text": "help",
            },
        )
    return control_left


def _SelectButtons(
        input_name: str,
        add_help: str,
        remove_help: str,
        up_help: str,
        down_help: str,
):
    """
    Create the select buttons for the file list.

    Needs alpine data with:
    - files: list of available files
    - filesStr: JSON string of the files
    - selected: list of selected files
    - isValid: boolean indicating if the selection is valid

    :return: The select buttons element.
    """
    with div(
        _class="control is-flex is-flex-direction-column is-justify-content-flex-start is-align-items-start ml-2"
    ) as control_right:

        with div(_class="is-flex"):
            # Upload button
            with button(
                _class="button is-success is-48x48 has-border",
                _title=add_help,
                **{
                    "@click.prevent": "openFileDialog"
                },
            ):
                i(_class="fa-solid fa-plus")
                input_(
                    _type="text",
                    _name=input_name,
                    _hidden=True,
                    **{
                        "x-model": "filesStr",
                    },
                )

            # Delete button
            with button(
                _class="button is-danger is-48x48 has-border ml-1",
                _title=remove_help,
                **{
                    "@click.prevent": """                
                        files = files.filter(file => !selected.includes(file));
                        selected = [];
                    """
                },
            ):
                i(_class="fa-solid fa-trash")

        # another set of buttons below
        with div(_class="is-flex mt-2"):
            # Up button
            with button(
                _class="button is-info is-48x48 has-border",
                _title=up_help,
                **{
                    "@click.prevent": """                
                        selected.forEach(file => {
                            const index = files.indexOf(file);
                            if (index > 0) {
                                // swap with the previous item
                                [files[index - 1], files[index]] = [files[index], files[index - 1]];
                            }
                        });
                        files = [...files]; // trigger reactivity
                    """
                },
            ):
                i(_class="fa-solid fa-arrow-up")

            # Down button
            with button(
                _class="button is-link is-outlined is-48x48 has-border ml-1",
                _title=down_help,
                **{
                    "@click.prevent": """                
                        selected.forEach(file => {
                            const index = files.indexOf(file);
                            if (index < files.length - 1) {
                                // swap with the next item
                                [files[index + 1], files[index]] = [files[index], files[index + 1]];
                                console.log('Moved down:', file);
                                console.log('Selected:', selected);
                            }
                        });
                        files = [...files]; // trigger reactivity
                    """
                },
            ):
                i(_class="fa-solid fa-arrow-down")

    return control_right


def FormFileList(
        label_text: str,
        input_name: str,
        validation_expr: str,
        multiple: bool = True,
        file_types: Sequence[str] | None = None,
        help_text: str = "",
        btn_add_help: str = "Add file",
        btn_remove_help: str = "Remove file",
        btn_up_help: str = "Move file up",
        btn_down_help: str = "Move file down",
        reverse: bool = False,
):
    """
    Create a file list form field with select and buttons.

    :param label_text: The label text for the field.
    :param input_name: The name attribute for the hidden input field.
    :param validation_expr: The expression to validate the selection.
    :param multiple: Whether to allow multiple file selection.
    :param file_types: The file types to filter in the file dialog. Format "description (*.<extension;*.extension;...)".
    :param help_text: The help text for the field.
    :param reverse: Whether to reverse the order of buttons and select box.

    :return: The form field element.
    """
    with div(
        cls="field is-horizontal is-full-width",
        **{
            "x-data": f"""{{     
                selected: [],
                help: `{help_text}`,
                filesStr: '',
                files: [],
                isValid: false,
                file_types: {format_py_to_js(file_types)},
                _validate_file(file) {{
                    if (this.file_types.length === 0) {{
                        return true; // no filter, accept all files
                    }}
                    const fileExt = file.split('.').pop().toLowerCase();
                    for (const type of this.file_types) {{
                        const file_types = type.split('(')[1].split(')')[0].split(';');
                        if (file_types.includes(`*.${{fileExt}}`) || file_types.includes(`*.*`)) {{
                            return true;
                        }}
                    }}
                    return false;
                }},
                _updateFiles(newFiles) {{
                    // extend files list, if newFiles is not already present
                    newFiles.forEach(file => {{
                        if (!this.files.includes(file) && this._validate_file(file)) {{
                            this.files.push(file);
                        }}
                    }});
                }},
                async openFileDialog() {{
                    const fileList = await pywebview.api.open_file_dialog({{                        
                        multiple: {format_py_to_js(multiple)},
                        file_types: this.file_types,
                    }});
                    this._updateFiles(fileList);
                }},
                init() {{
                    this.$watch('files', value => {{     
                        this.filesStr = JSON.stringify(value);

                        this.isValid = {validation_expr};
                        const parentForm = this.$el.closest('form[x-data]');
                        if(parentForm){{
                            const parentData = Alpine.$data(parentForm);
                            parentData.updateValidity();
                        }} else {{
                            console.log('No parent form found');
                        }}
                    }});   
                    // watch alpine store 'drag_drop' for dropped files
                    this.$watch('$store.drag_drop.files', value => {{
                        this._updateFiles(value);
                    }});
                }},
            }}"""
        },
    ) as field:
        # Field label
        with div(_class="field-label"):
            label(label_text, _class="label")

        # Field body
        with div(_class="field-body is-flex is-flex-grow-5"):
            if not reverse:
                # Left-side file list
                _SelectBox()
                # Right-side controls
                _SelectButtons(
                    input_name=input_name,
                    add_help=btn_add_help,
                    remove_help=btn_remove_help,
                    up_help=btn_up_help,
                    down_help=btn_down_help,
                )
            else:
                # Left-side controls
                _SelectButtons(
                    input_name=input_name,
                    add_help=btn_add_help,
                    remove_help=btn_remove_help,
                    up_help=btn_up_help,
                    down_help=btn_down_help,
                )
                # Right-side file list
                _SelectBox()
    return field


__all__ = [
    'FormFileList',
]



# src/file_conversor/backend/gui/_api/audio/convert.py

from flask import json, render_template, request, url_for
from pathlib import Path
from typing import Any

# user-provided modules
from file_conversor.cli.audio._ffmpeg_cmd import ffmpeg_audio_run, EXTERNAL_DEPENDENCIES

from file_conversor.backend.gui.flask_api import FlaskApi
from file_conversor.backend.gui.flask_api_status import FlaskApiStatus

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


def _api_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """Thread to handle audio converting."""
    logger.debug(f"Audio convert thread received: {params}")

    input_files = [Path(i) for i in params['input-files']]
    file_format = str(params['file-format'])
    audio_bitrate = int(params['audio-bitrate'])
    output_dir = Path(params['output-dir'])

    logger.info(f"[bold]{_('Converting audio files')}[/]...")
    ffmpeg_audio_run(
        input_files=input_files,
        file_format=file_format,
        audio_bitrate=audio_bitrate,
        output_dir=output_dir,
        progress_callback=lambda p, pm: status.set_progress(pm.update_progress(p)),
    )

    logger.debug(f"{status}")


def api_audio_convert():
    """API endpoint to convert audio files."""
    logger.info(f"[bold]{_('Audio convert requested via API.')}[/]")
    return FlaskApi.execute_response(_api_thread)


__all__ = [
    "api_audio_convert",
    "EXTERNAL_DEPENDENCIES",
]



# src\file_conversor\backend\pikepdf_backend.py

"""
This module provides functionalities for handling PDF files using ``pikepdf`` backend (qpdf python wrapper).
"""

import pikepdf

from pikepdf import ObjectStreamMode

from pathlib import Path
from typing import Any, Callable

# user-provided imports
from file_conversor.config.log import Log

from file_conversor.backend.abstract_backend import AbstractBackend

LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class PikePDFBackend(AbstractBackend):
    """
    A class that provides an interface for handling PDF files using ``pikepdf`` (qpdf python wrapper).
    """

    SUPPORTED_IN_FORMATS = {
        "pdf": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "pdf": {},
    }
    EXTERNAL_DEPENDENCIES: set[str] = set()

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the ``pikepdf`` backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    @staticmethod
    def is_encrypted(file_path: str | Path) -> bool:
        """Checks if PDF file is encrypted"""
        try:
            with pikepdf.open(file_path):
                return False  # opened without password → not encrypted
        except pikepdf.PasswordError:
            return True  # opening failed because it's encrypted

    def compress(
        self,
        input_file: str | Path,
        output_file: str | Path,
        progress_callback: Callable[[int], Any] | None = None,
        decrypt_password: str | None = None,
        linearize: bool = True,
    ):
        """
        Compress input PDF file.

        :param input_files: Input PDF file. 
        :param output_files: Output PDF file.
        :param progress_callback: Progress callback executed as PDF is processed. Format callback(0-100). Defaults to None (no progress callback).
        :param decryption_password: Decryption password for input PDF file. Defaults to None (do not decrypt).
        :param linearize: Linearize PDF file structures (speed up web render). Defaults to True.

        :raises FileNotFoundError: if input file not found.
        :raises PDFError, ForeignObjectError: if qpdf errors.
        """
        self.check_file_exists(input_file)

        # Open PDF (with password if provided)
        preserve_encryption = True if decrypt_password and self.is_encrypted(input_file) else None
        with pikepdf.open(input_file, password=decrypt_password if decrypt_password else "") as pdf:
            # Save optimized PDF
            pdf.save(output_file,
                     progress=progress_callback,  # callback(0-100)
                     encryption=preserve_encryption,  # preserve encryption
                     compress_streams=True,  # compress streams
                     object_stream_mode=ObjectStreamMode(ObjectStreamMode.generate),  # generate streams as needed (max compression)
                     linearize=linearize,
                     )


__all__ = [
    "PikePDFBackend",
]



# src\file_conversor\backend\office\excel_backend.py

from pathlib import Path
from typing import Any, Callable

# user-provided imports
from file_conversor.config import Log
from file_conversor.config.locale import get_translation
from file_conversor.backend.office.abstract_msoffice_backend import AbstractMSOfficeBackend, Win32Com

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class ExcelBackend(AbstractMSOfficeBackend):
    """
    A class that provides an interface for handling doc files using ``excel`` (comtypes).
    """

    SUPPORTED_IN_FORMATS = {
        "xls": {},
        "xlsx": {},
        "ods": {},
    }
    SUPPORTED_OUT_FORMATS = {
        # format = xlFormat VBA code
        # https://learn.microsoft.com/en-us/office/vba/api/excel.xlfileformat
        "xls": {'format': 56},
        "xlsx": {'format': 51},
        "ods": {'format': 60},
        "csv": {'format': 6},
        "pdf": {'format': 57},
        "html": {'format': 44},
    }
    EXTERNAL_DEPENDENCIES = set()

    def __init__(
        self,
        install_deps: bool | None = None,
        verbose: bool = False,
    ):
        """
        Initialize the backend

        :param install_deps: Reserved for future use. 
        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__(
            prog_id="Excel.Application",
            install_deps=install_deps,
            verbose=verbose,
        )

    def convert(
        self,
        files: list[tuple[Path, Path]],
        file_processed_callback: Callable[[Path], Any] | None = None,
    ):
        """
        Convert input file into an output file.

        :param files: List of tuples containing input and output file paths.

        :raises FileNotFoundError: if input file not found.
        :raises RuntimeError: if os != Windows.
        """
        with Win32Com(self.PROG_ID, visible=False) as excel:
            for input_file, output_file in files:
                input_path = Path(input_file).resolve()

                output_path = Path(output_file).resolve()
                output_path = output_path.with_suffix(output_path.suffix.lower())

                self.check_file_exists(str(input_path))

                out_ext = output_path.suffix[1:]
                out_config = ExcelBackend.SUPPORTED_OUT_FORMATS[out_ext]

                workbook = excel.Workbooks.Open(str(input_path))
                if output_path.suffix.lower() == ".pdf":
                    workbook.ExportAsFixedFormat(
                        Filename=str(output_path),
                        Type=0,  # = pdf
                        Quality=0,
                        IncludeDocProperties=True,
                        IgnorePrintAreas=False,
                        OpenAfterPublish=False,
                    )
                else:
                    workbook.SaveAs(
                        str(output_path),
                        FileFormat=out_config['format'],
                    )
                workbook.Close(SaveChanges=False)

                if file_processed_callback:
                    file_processed_callback(input_path)


__all__ = [
    "ExcelBackend",
]



# src\file_conversor\backend\gui\_components\media.py

from typing import Any

from file_conversor.utils.dominate_utils import div


def Media(
        content: Any | None = None,
        left: Any = None,
        right: Any = None,
        **kwargs
):
    with div(_class="media", **kwargs) as media:
        if left:
            with div(_class="media-left") as media_left:
                media_left.add(left)
        if right:
            with div(_class="media-right") as media_right:
                media_right.add(right)
        if content:
            with div(_class="media-content") as media_content:
                media_content.add(content)
    return media


__all__ = [
    'Media',
]



# src\file_conversor\backend\image\__init__.py

from file_conversor.backend.image.compress_backend import *
from file_conversor.backend.image.img2pdf_backend import *
from file_conversor.backend.image.pillow_backend import *
from file_conversor.backend.image.pymusvg_backend import *



# src\file_conversor\backend\abstract_backend.py

"""
This module provides functionalities for handling external backends.
"""

import os
import platform
import shutil
import typer

from typing import Iterable
from pathlib import Path
from rich import print

# user-provided imports
from file_conversor.dependency import AbstractPackageManager

from file_conversor.config import Log
from file_conversor.config.locale import get_translation

LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class AbstractBackend:
    """
    Class that provides an interface for handling internal/external backends.
    """

    @staticmethod
    def find_in_path(name: str | Path) -> Path:
        """
        Finds name path in PATH env

        :return: Path for name

        :raises FileNotFoundError: if name not found
        """
        path_str = shutil.which(name)
        if not path_str:
            raise FileNotFoundError(f"'{name}' {_('not found in PATH environment')}")
        path = Path(path_str).resolve()
        logger.info(f"'{name}' {_('found')}: {path}")
        return path

    @staticmethod
    def check_file_exists(filename: str | Path):
        """
        Check if `filename` exists

        :raises FileNotFoundError: if file not found
        """
        if not os.path.isfile(filename):
            raise FileNotFoundError(f"{_("File")} '{filename}' {_("not found")}")

    def __init__(
        self,
        pkg_managers: set[AbstractPackageManager] | None = None,
        install_answer: bool | None = None,
    ):
        """
        Initialize the abstract backend.

        Checks if external dependencies are installed, and if not, install them.

        :param pkg_managers: Pkg managers configured to install external dependencies. Defaults to None (no external dependency required).
        :param install_answer: If True, do not ask user to install dependency (auto install). If False, do not install missing dependencies. If None, ask user for action. Defaults to None.

        :raises RuntimeError: Cannot install missing dependency or unknown OS detected.
        """
        super().__init__()
        self._pkg_managers = pkg_managers if pkg_managers else set()
        self._install_answer = install_answer

        self.verify_missing_deps()

    def verify_missing_deps(self):
        """
        Verify and install missing external dependencies, if they are not installed.
        :raises RuntimeError: Cannot install missing dependency.
        """
        # identify OS and package manager
        os_type = platform.system()
        for pkg_mgr in self._pkg_managers:
            if os_type not in pkg_mgr.get_supported_oses():
                continue

            # supported pkg manager found, proceed to check for dependencies
            missing_deps = self._check_missing_deps(pkg_mgr)
            if not missing_deps:
                continue

            # install package manager, if not present already
            self._check_pkg_manager_installed(pkg_mgr)

            # install missing dependencies
            self._install_missing_deps(pkg_mgr, missing_deps)

    def _check_missing_deps(self, pkg_mgr: AbstractPackageManager) -> set[str]:
        """
        Check for missing dependencies using the provided package manager.

        :param pkg_mgr: Package manager to use for checking dependencies.
        :return: Set of missing dependencies.
        """
        missing_deps = pkg_mgr.check_dependencies()
        if missing_deps:
            logger.warning(f"[bold]{_("Missing dependencies detected")}[/]: {", ".join(missing_deps)}")
        return missing_deps

    def _check_pkg_manager_installed(self, pkg_mgr: AbstractPackageManager):
        """
        Check if package manager is installed, and if not, install it.

        :param pkg_mgr: Package manager to check/install.
        :raises RuntimeError: Cannot install missing package manager.
        """
        pkg_mgr_bin = pkg_mgr.get_pkg_manager_installed()
        if pkg_mgr_bin:
            logger.info(f"Package manager found in '{pkg_mgr_bin}'")
            return

        logger.warning(f"{_('Package manager not found, installing it ...')}.")
        user_prompt = self._install_answer
        if user_prompt is None:
            user_prompt = typer.confirm(
                _("Install package manager for the current user?"),
                default=True,
            )
        if not user_prompt:
            raise RuntimeError(_("Cannot install missing package manager"))

        pkg_mgr_path = pkg_mgr.install_pkg_manager()
        logger.info(f"Package manager installed in '{pkg_mgr_path}'")
        logger.info(f"[bold]{_("Package Manager Installation")}[/]: [green]{_("SUCCESS")}[/]")

    def _install_missing_deps(self, pkg_mgr: AbstractPackageManager, missing_deps: Iterable[str]):
        """
        Install missing dependencies using the provided package manager.

        :param pkg_mgr: Package manager to use for installation.
        :param missing_deps: Set of missing dependencies to install.
        :raises RuntimeError: Cannot install missing dependencies.
        """

        user_prompt = self._install_answer
        if user_prompt is None:
            user_prompt = typer.confirm(
                _("Install missing dependencies for the current user?"),
                default=True,
            )
        if not user_prompt:
            raise RuntimeError(_("Cannot install missing dependencies"))

        pkg_mgr.install_dependencies(missing_deps)
        logger.info(f"[bold]{_("External Dependencies Installation")}[/]: [green]{_("SUCCESS")}[/]")


__all__ = [
    "AbstractBackend",
]



# src/file_conversor/backend/gui/hash/create.py

from pathlib import Path
from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.hash_backend import HashBackend

from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.hash._dom_page import hash_create_nav_item, hash_index_nav_item

from file_conversor.utils.bulma_utils import InputFilesField, OutputFileField
from file_conversor.utils.dominate_bulma import PageForm

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageHashCreate():
    return PageForm(
        InputFilesField(),
        OutputFileField(
            *[
                (f, f'{f.upper()} {_("file")}')
                for f in HashBackend.SUPPORTED_OUT_FORMATS
            ],
            current_value="output.sha256",
        ),
        api_endpoint=f"{url_for('api_hash_create')}",
        nav_items=[
            home_nav_item(),
            hash_index_nav_item(),
            hash_create_nav_item(active=True),
        ],
        _title=f"{_('Hash Create')} - File Conversor",
    )


def hash_create():
    return render_template_string(str(
        PageHashCreate()
    ))


__all__ = [
    'hash_create',
]



# src\file_conversor\backend\gui\_webview_api.py

import re
import webview

from webview.dom import DOMEventHandler
from pathlib import Path
from typing import Any, Sequence
from natsort import natsorted, ns


# user-provided modules
from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

from file_conversor.utils.formatters import format_file_types_webview, format_py_to_js

from file_conversor.system import set_window_icon

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


class WebViewState:
    """State shared between the webview and the main application."""

    # Global last open directory
    _lastOpenDir: Path = Path(CONFIG['gui-output-dir']).resolve()
    _icon_configured: bool = False

    @classmethod
    def is_icon_configured(cls) -> bool:
        return cls._icon_configured

    @classmethod
    def set_icon_configured(cls, value: bool) -> None:
        cls._icon_configured = value

    @classmethod
    def get_last_open_dir(cls) -> Path:
        return cls._lastOpenDir

    @classmethod
    def set_last_open_dir(cls, path: str | Path) -> None:
        if isinstance(path, str):
            path = Path(path)
        cls._lastOpenDir = path.resolve()


class WebViewAPI:
    """API exposed to the webview JavaScript context."""

    def _get_window(self, index: int = 0) -> webview.Window:
        """Get the webview window."""
        if len(webview.windows) > index:
            window = webview.windows[index]
            # logger.debug(f"Found webview windows: {','.join([w.title for w in webview.windows])}")
            return window
        raise RuntimeError(_("No webview window found."))

    # ###### #
    # EVENTS #
    # ###### #

    def _on_load(self) -> None:
        logger.debug("WebViewAPI page loaded.")
        window = self._get_window()
        window.dom.document.events.dragover += DOMEventHandler(
            self._on_drag,
            prevent_default=True,
            stop_propagation=True,
            stop_immediate_propagation=True,
            debounce=100,
        )  # pyright: ignore[reportOperatorIssue]
        window.dom.document.events.drop += DOMEventHandler(
            self._on_drop,
            prevent_default=True,
            stop_propagation=True,
            stop_immediate_propagation=True,
        )  # pyright: ignore[reportOperatorIssue]

    def _on_drag(self, event) -> None:
        # empty handler to allow drop event
        pass

    def _on_drop(self, event) -> None:
        logger.debug("Drop event detected")
        data_transfer: dict[str, Any] = event.get('dataTransfer', {})
        files: list[dict[str, Any]] = data_transfer.get('files', [])

        filepaths: list[str] = [
            str(file.get('pywebviewFullPath'))
            for file in files
            if file.get('pywebviewFullPath')
        ]
        filepaths = natsorted(filepaths, alg=ns.PATH)  # natural sort for paths

        if filepaths:
            dir_name = Path(filepaths[0]).parent
            WebViewState.set_last_open_dir(dir_name)

        logger.debug(f"Dropped files: {filepaths}")
        self._get_window().evaluate_js("""
            window.dispatchEvent(new CustomEvent('filesDropped', {
                detail: { files: %s }
            }));
        """ % format_py_to_js(filepaths))

    # ######### #
    # COMMANDS #
    # ######### #

    def reset_config(self):
        """Reset the application configuration to factory defaults."""
        CONFIG.reset_factory_default()
        CONFIG.save()
        logger.info("Configuration reset to factory defaults.")

    def get_config(self) -> dict[str, Any]:
        """Get the current application configuration."""
        return CONFIG.to_dict()

    def touch_file(self, options: dict[str, Any]) -> bool:
        """Create an empty file at the specified path."""
        path = options.get("path")
        if not path:
            raise ValueError("Path must be provided.")

        Environment.touch(path)
        logger.debug(f"Touched file at '{path}'.")
        return True

    def move_file(self, options: dict[str, Any]) -> bool:
        """Move a file from src to dst."""
        src = options.get("src")
        dst = options.get("dst")
        overwrite = bool(options.get("overwrite", False))

        if not src or not dst:
            raise ValueError("Source and destination paths must be provided.")

        Environment.move(src, dst, overwrite=overwrite)
        logger.debug(f"Moved file from '{src}' to '{dst}' (overwrite={overwrite}).")
        return True

    def copy_file(self, options: dict[str, Any]) -> bool:
        """Copy a file from src to dst."""
        src = options.get("src")
        dst = options.get("dst")
        overwrite = bool(options.get("overwrite", False))

        if not src or not dst:
            raise ValueError("Source and destination paths must be provided.")

        Environment.copy(src, dst, overwrite=overwrite)
        logger.debug(f"Copied file from '{src}' to '{dst}' (overwrite={overwrite}).")
        return True

    def close(self) -> bool:
        self._get_window().destroy()
        logger.debug("Window closed.")
        return True

    def minimize(self) -> bool:
        self._get_window().minimize()
        logger.debug("Window minimized.")
        return True

    def maximize(self) -> bool:
        self._get_window().maximize()
        logger.debug("Window maximized.")
        return True

    def show(self) -> bool:
        self._get_window().show()
        logger.debug("Window shown.")
        return True

    def hide(self) -> bool:
        self._get_window().hide()
        logger.debug("Window hidden.")
        return True

    def resize(self, options: dict[str, int]) -> bool:
        width = options.get("width")
        height = options.get("height")

        if not width or not height:
            raise ValueError("Width and height must be provided.")

        self._get_window().resize(int(width), int(height))
        logger.debug(f"Window resized to {width}x{height}.")
        return True

    def fullscreen(self) -> bool:
        self._get_window().toggle_fullscreen()
        logger.debug("Window fullscreen toggled.")
        return True

    def move(self, options: dict[str, int]) -> bool:
        x = options.get("x")
        y = options.get("y")

        if not x or not y:
            raise ValueError("X and Y coordinates must be provided.")

        self._get_window().move(int(x), int(y))
        logger.debug(f"Window moved to ({x},{y}).")
        return True

    def set_title(self, options: dict[str, str]) -> bool:
        title = options.get("title")

        if not title:
            raise ValueError("Title must be provided.")

        self._get_window().set_title(title)
        logger.debug(f"Window title set to '{title}'.")
        return True

    def set_icon(self) -> None:
        """Set the window icon (Windows only)."""
        if WebViewState.is_icon_configured():
            return
        try:
            set_window_icon(
                self._get_window().title,
                icon_path=Environment.get_app_icon(),
                cx=128, cy=128,
            )
            WebViewState.set_icon_configured(True)
            logger.info("Window icon set.")
        except Exception as e:
            logger.warning(f"Failed to set window icon - {repr(e)}")

    def get_last_open_dir(self) -> str:
        """Get the last opened directory."""
        return str(WebViewState.get_last_open_dir())

    def open_folder_dialog(self, options: dict[str, Any]) -> list[str]:
        """
        Open a folder in the system file explorer.

        :param multiple: Whether to allow multiple folder selection.
        :param path: Optional initial directory path.
        """
        window = self._get_window()
        result = list(window.create_file_dialog(
            dialog_type=webview.FileDialog.FOLDER,
            directory=options.get("path") or self.get_last_open_dir(),
            allow_multiple=bool(options.get("multiple", False)),
        ) or [])
        result = natsorted(result, alg=ns.PATH)  # natural sort for paths
        if result:
            dir_path = Path(result[0])
            WebViewState.set_last_open_dir(dir_path)
        logger.debug(f"Selected save file: {result}")
        return result

    def open_file_dialog(self, options: dict[str, Any]) -> list[str]:
        """
        Open a file dialog and return the selected file paths.

        :param file_types: Optional file type filters (e.g., 'Image Files (*.png;*.jpg)')
        :param multiple: Whether to allow multiple file selection.
        :param path: Optional initial directory path.

        :return: List of selected file paths.
        """
        window = self._get_window()
        result = list(window.create_file_dialog(
            dialog_type=webview.FileDialog.OPEN,
            directory=options.get("path") or self.get_last_open_dir(),
            allow_multiple=bool(options.get("multiple", False)),                  # allow selecting multiple files
            file_types=options.get("file_types") or [
                format_file_types_webview(),  # filter for all files
            ],
        ) or [])
        result = natsorted(result, alg=ns.PATH)  # natural sort for paths
        if result:
            dir_path = Path(result[0]).parent
            WebViewState.set_last_open_dir(dir_path)
        logger.debug(f"Selected files: {result}")
        return result

    def save_file_dialog(self, options: dict[str, Any]) -> list[str]:
        """
        Open a save file dialog and return the selected file path.

        :param file_types: Optional file type filters (e.g., 'Text Files (*.txt)')
        :param filename: Optional default file name to use in the dialog.
        :param path: Optional initial directory path.

        :return: The selected file path.
        """
        window = self._get_window()
        result = list(window.create_file_dialog(
            dialog_type=webview.FileDialog.SAVE,
            directory=options.get("path") or self.get_last_open_dir(),
            save_filename=Path(options.get("filename", "")).name,
            file_types=options.get("file_types", [format_file_types_webview()]),
        ) or [])
        result.sort()

        logger.debug(f"Selected save file: {result}")
        if not result:
            return result

        dir_path = Path(result[0]).parent
        WebViewState.set_last_open_dir(dir_path)

        # adjust file suffix based on selected file type
        file_types: list[str] = options.get("file_types", [])
        if not file_types:
            return result

        match = re.search(r"\(\*(\.[^);]+)", file_types[0])
        if not match:
            return result

        suffix = match.group(1)
        res = f"{Path(result[0]).with_suffix(suffix).resolve()}"
        logger.debug(f"Adjusted saved file: '{res}'")
        return [res]


__all__ = ['WebViewAPI']



# src\file_conversor\cli\pipeline\_info.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "pipeline"

# SUBCOMMANDS
CREATE_NAME = "create"
EXECUTE_NAME = "execute"

__all__ = [
    "COMMAND_NAME",

    "CREATE_NAME",
    "EXECUTE_NAME",
]



# src\file_conversor\config\cleanup.py

import atexit
import sys

from typing import Callable

# user provided imports
from file_conversor.config.log import Log

# Get app config
logger = Log.get_instance().getLogger(__name__)


__cleanup_tasks: list[Callable[[], None]] = []


def add_cleanup_task(func: Callable[[], None]) -> None:
    """Add a cleanup task to be executed on exit."""
    __cleanup_tasks.append(func)


def __cleanup():
    """Run all registered cleanup tasks."""
    logger.info("Running cleanup tasks ...")
    for task in __cleanup_tasks:
        try:
            task()
        except Exception as e:
            logger.error(f"Error in cleanup task {task}: {repr(e)}", exc_info=True)


atexit.register(__cleanup)


__all__ = [
    "add_cleanup_task",
]



# src/file_conversor/backend/gui/_api/image/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui._api.image.antialias import api_image_antialias
from file_conversor.backend.gui._api.image.blur import api_image_blur
from file_conversor.backend.gui._api.image.compress import api_image_compress
from file_conversor.backend.gui._api.image.convert import api_image_convert
from file_conversor.backend.gui._api.image.enhance import api_image_enhance
from file_conversor.backend.gui._api.image.filter import api_image_filter
from file_conversor.backend.gui._api.image.info import api_image_info
from file_conversor.backend.gui._api.image.mirror import api_image_mirror
from file_conversor.backend.gui._api.image.render import api_image_render
from file_conversor.backend.gui._api.image.resize import api_image_resize
from file_conversor.backend.gui._api.image.rotate import api_image_rotate
from file_conversor.backend.gui._api.image.to_pdf import api_image_to_pdf
from file_conversor.backend.gui._api.image.unsharp import api_image_unsharp


def routes():
    return [
        FlaskRoute(
            rule="/api/image/antialias",
            handler=api_image_antialias,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/blur",
            handler=api_image_blur,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/compress",
            handler=api_image_compress,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/convert",
            handler=api_image_convert,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/enhance",
            handler=api_image_enhance,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/filter",
            handler=api_image_filter,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/info",
            handler=api_image_info,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/mirror",
            handler=api_image_mirror,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/render",
            handler=api_image_render,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/resize",
            handler=api_image_resize,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/rotate",
            handler=api_image_rotate,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/to_pdf",
            handler=api_image_to_pdf,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/image/unsharp",
            handler=api_image_unsharp,
            methods=["POST"],
        ),
    ]


__all__ = ['routes']



# src\file_conversor\cli\pdf\__init__.py

import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.pdf._typer import COMMAND_NAME

from file_conversor.cli.pdf.compress_cmd import typer_cmd as compress_cmd
from file_conversor.cli.pdf.convert_cmd import typer_cmd as convert_cmd
from file_conversor.cli.pdf.decrypt_cmd import typer_cmd as decrypt_cmd
from file_conversor.cli.pdf.encrypt_cmd import typer_cmd as encrypt_cmd
from file_conversor.cli.pdf.extract_cmd import typer_cmd as extract_cmd
from file_conversor.cli.pdf.extract_img_cmd import typer_cmd as extract_img_cmd
from file_conversor.cli.pdf.merge_cmd import typer_cmd as merge_cmd
from file_conversor.cli.pdf.ocr_cmd import typer_cmd as ocr_cmd
from file_conversor.cli.pdf.repair_cmd import typer_cmd as repair_cmd
from file_conversor.cli.pdf.rotate_cmd import typer_cmd as rotate_cmd
from file_conversor.cli.pdf.split_cmd import typer_cmd as split_cmd

_ = get_translation()

pdf_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("PDF file manipulation"),
)
# SECURITY_PANEL
pdf_cmd.add_typer(encrypt_cmd)
pdf_cmd.add_typer(decrypt_cmd)

# TRANSFORMATION_PANEL
pdf_cmd.add_typer(compress_cmd)
pdf_cmd.add_typer(extract_cmd)
pdf_cmd.add_typer(merge_cmd)
pdf_cmd.add_typer(rotate_cmd)
pdf_cmd.add_typer(split_cmd)
pdf_cmd.add_typer(ocr_cmd)

# OTHERS_PANEL
pdf_cmd.add_typer(convert_cmd)
pdf_cmd.add_typer(extract_img_cmd)
pdf_cmd.add_typer(repair_cmd)

__all__ = [
    "pdf_cmd",
]



# src/file_conversor/backend/gui/_components/_class.py

def class_is_pulled_left():
    return "is-pulled-left"


def class_is_pulled_right():
    return "is-pulled-right"


def class_is_clickable():
    return "is-clickable"


def class_is_unselectable():
    return "is-unselectable"


def class_is_relative():
    return "is-relative"


__all__ = [
    'class_is_pulled_left',
    'class_is_pulled_right',
    'class_is_clickable',
    'class_is_unselectable',
    'class_is_relative',
]



# src/file_conversor/backend/gui/config/_tab_network.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.utils.dominate_bulma import FormFieldHorizontal, FormFieldInput

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation, AVAILABLE_LANGUAGES

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def TabConfigNetwork() -> tuple | list:
    return (
        FormFieldHorizontal(
            FormFieldInput(
                validation_expr="Number.parseInt(value) >= 1 && Number.parseInt(value) <= 65535",
                current_value=CONFIG['port'],
                _name="port",
                _type="number",
                help=_("Set the port number for the application to listen on (1-65535)."),
            ),
            label_text=_("Port"),
        ),
    )


__all__ = ['TabConfigNetwork']



# src\file_conversor\backend\__init__.py

"""
Initialization module for the backend package.

This module imports all functionalities from backend wrappers,
making them available when importing the backend package.
"""

# SUBMODULES
from file_conversor.backend.audio_video import *
from file_conversor.backend.ebook import *
from file_conversor.backend.image import *
from file_conversor.backend.office import *
from file_conversor.backend.pdf import *

# OTHER BACKENDS
from file_conversor.backend.batch_backend import *
from file_conversor.backend.git_backend import *
from file_conversor.backend.hash_backend import *
from file_conversor.backend.http_backend import *
from file_conversor.backend.text_backend import *
from file_conversor.backend.win_reg_backend import *



# src/file_conversor/backend/gui/_components/typography.py

class TextSize:
    SIZE_1 = "is-size-1"
    SIZE_2 = "is-size-2"
    SIZE_3 = "is-size-3"
    SIZE_4 = "is-size-4"
    SIZE_5 = "is-size-5"
    SIZE_6 = "is-size-6"
    SIZE_7 = "is-size-7"


class TextAlign:
    LEFT = "has-text-left"
    CENTERED = "has-text-centered"
    RIGHT = "has-text-right"
    JUSTIFIED = "has-text-justified"


class TextTransform:
    UPPERCASE = "is-uppercase"
    LOWERCASE = "is-lowercase"
    CAPITALIZED = "is-capitalized"
    UNDERLINE = "is-underlined"
    ITALIC = "is-italic"


class TextWeight:
    LIGHT = "has-text-weight-light"
    NORMAL = "has-text-weight-normal"
    MEDIUM = "has-text-weight-medium"
    SEMIBOLD = "has-text-weight-semibold"
    BOLD = "has-text-weight-bold"
    EXTRABOLD = "has-text-weight-extrabold"


class TextFont:
    MONOSPACE = "is-family-monospace"
    SANS_SERIF = "is-family-sans-serif"
    PRIMARY = "is-family-primary"
    SECONDARY = "is-family-secondary"
    CODE = "is-family-code"


__all__ = [
    'TextSize',
    'TextAlign',
    'TextTransform',
    'TextWeight',
    'TextFont',
]




# src\file_conversor\cli\text\check_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print


# user-provided modules
from file_conversor.backend import TextBackend

from file_conversor.cli.text._typer import COMMAND_NAME, CHECK_NAME
from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument


# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = TextBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    for ext in TextBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="check",
                description="Check",
                command=f'cmd.exe /k "{Environment.get_executable()} "{COMMAND_NAME}" "{CHECK_NAME}" "%1""',
                icon=str(icons_folder_path / 'check.ico'),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_text_check_cmd(
    input_files: List[Path],
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    text_backend = TextBackend(verbose=STATE["verbose"])
    logger.info(f"{_('Checking files')} ...")

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        text_backend.check(
            input_file=input_file,
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=Path(), overwrite=True)  # overwrite is True because no output files are generated
    cmd_mgr.run(callback)

    logger.info(f"{_('Check')}: [bold green]{_('SUCCESS')}[/].")


# text check
@typer_cmd.command(
    name=CHECK_NAME,
    help=f"""
        {_('Checks a text file schema compliance (json, xml, yaml, etc).')}        
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor {COMMAND_NAME} {CHECK_NAME} file.json` 

- `file_conversor {COMMAND_NAME} {CHECK_NAME} file1.json file2.yaml` 
""")
def check(
    input_files: Annotated[List[Path], InputFilesArgument(TextBackend)],
):
    execute_text_check_cmd(
        input_files=input_files,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_text_check_cmd",
]



# src\file_conversor\config\state.py

from pathlib import Path

from typing import Any

# user provided imports
from file_conversor.config.log import Log

from file_conversor.config.abstract_singleton_thread_safe import AbstractSingletonThreadSafe

# Get app config
LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


# STATE ACTIONS
def disable_log(value):
    if not value:
        return
    logger.info(f"'File logging': [blue red]'DISABLED'[/]")
    LOG.set_dest_folder(None)


def disable_progress(value):
    if not value:
        return
    logger.info(f"Progress bars: [blue red]DISABLED[/]")


def enable_quiet_mode(value):
    if not value:
        return
    logger.info(f"Quiet mode: [blue bold]ENABLED[/]")
    LOG.set_level(Log.LEVEL_ERROR)


def enable_verbose_mode(value):
    if not value:
        return
    logger.info(f"Verbose mode: [blue bold]ENABLED[/]")
    LOG.set_level(Log.LEVEL_INFO)


def enable_debug_mode(value):
    if not value:
        return
    logger.info(f"Debug mode: [blue bold]ENABLED[/]")
    LOG.set_level(Log.LEVEL_DEBUG)


def enable_overwrite_output_mode(value):
    if not value:
        return
    logger.info(f"Output overwrite mode: [blue bold]ENABLED[/]")


# STATE controller dict class
class State(AbstractSingletonThreadSafe):
    def __init__(self) -> None:
        super().__init__()
        self.__init_state()

    def __init_state(self):
        # Define state dictionary
        self.__data = {
            # app options
            "no-log": False,
            "no-progress": False,
            "quiet": False,
            "verbose": False,
            "debug": False,
            "overwrite-output": False,
        }
        self.__callbacks = {
            "no-log": disable_log,
            "no-progress": disable_progress,
            "quiet": enable_quiet_mode,
            "verbose": enable_verbose_mode,
            "debug": enable_debug_mode,
            "overwrite-output": enable_overwrite_output_mode,
        }
        # run callbacks
        for key, value in self.__data.items():
            self._run_callbacks(key=key, value=value)

    def _run_callbacks(self, key: str, value):
        if key in self.__callbacks:
            self.__callbacks[key](value)

    def __repr__(self) -> str:
        return repr(self.__data)

    def __str__(self) -> str:
        return str(self.__data)

    def __getitem__(self, key) -> Any:
        if key not in self.__data:
            raise KeyError(f"Key '{key}' not found in STATE")
        return self.__data[key]

    def __setitem__(self, key, value):
        if key not in self.__data:
            raise KeyError(f"Key '{key}' is not a valid key for STATE. Valid options are {', '.join(self.__data.keys())}")
        self.__data[key] = value

        # run callback
        self._run_callbacks(key=key, value=value)

    def __contains__(self, key) -> bool:
        return key in self.__data

    def __len__(self) -> int:
        return len(self.__data)

    def to_dict(self) -> dict[str, Any]:
        return self.__data.copy()

    def update(self, new: dict):
        for key, value in new.items():
            self[key] = value


__all__ = [
    "State",
]




# src\file_conversor\cli\video\_ffmpeg_cmd.py

from rich import print

from typing import Annotated, Any, Callable, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend
from file_conversor.backend.audio_video.ffmpeg_filter import FFmpegFilter

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


EXTERNAL_DEPENDENCIES = FFmpegBackend.EXTERNAL_DEPENDENCIES


def ffmpeg_audio_run(  # pyright: ignore[reportUnusedFunction]
    input_files: List[Path],

    file_format: str,
    out_stem: str = "",

    audio_bitrate: int = 0,
    audio_codec: str | None = None,

    output_dir: Path = Path(),
    progress_callback: Callable[[float, ProgressManager], Any] | None = None,
):
    # init ffmpeg
    ffmpeg_backend = FFmpegBackend(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
        overwrite_output=STATE["overwrite-output"],
    )

    # set filters
    audio_filters: list[FFmpegFilter] = []

    two_pass = (audio_bitrate > 0)

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        ffmpeg_backend.set_files(input_file=input_file, output_file=output_file)
        ffmpeg_backend.set_audio_codec(codec=audio_codec, bitrate=audio_bitrate, filters=audio_filters)

        progress_update_cb = progress_mgr.update_progress
        if progress_callback is not None:
            def progress_update_cb(step_progress: float): return progress_callback(step_progress, progress_mgr)  # pyright: ignore[reportOptionalCall]

        progress_complete_cb = progress_mgr.complete_step
        if progress_callback is not None:
            def progress_complete_cb(): return progress_callback(progress_mgr.complete_step(), progress_mgr)  # pyright: ignore[reportOptionalCall]

        # display current progress
        ffmpeg_backend.execute(
            progress_callback=progress_update_cb,
            pass_num=1 if two_pass else 0,
        )
        progress_complete_cb()

        if two_pass:
            # display current progress
            ffmpeg_backend.execute(
                progress_callback=progress_update_cb,
                pass_num=2,
            )
            progress_complete_cb()

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, steps=2 if two_pass else 1, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_suffix=f".{file_format}", out_stem=out_stem)

    logger.info(f"{_('FFMpeg result')}: [green][bold]{_('SUCCESS')}[/bold][/green]")


__all__ = [
    "EXTERNAL_DEPENDENCIES",
    "ffmpeg_audio_run",
]




# src\file_conversor\cli\video\list_formats_cmd.py

import typer

from rich import print

from typing import Annotated, List
from pathlib import Path

# user-provided modules
from file_conversor.backend import FFmpegBackend

from file_conversor.cli.video._typer import OTHERS_PANEL as RICH_HELP_PANEL
from file_conversor.cli.video._typer import COMMAND_NAME, LIST_FORMATS_NAME

from file_conversor.config import Environment, Configuration, State, Log, get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.validators import check_valid_options
from file_conversor.utils.typer_utils import FormatOption

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = FFmpegBackend.EXTERNAL_DEPENDENCIES


@typer_cmd.command(
    name=LIST_FORMATS_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('List available video formats and codecs.')}

        {_('If a video format is provided, only codecs for that format will be shown.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor {COMMAND_NAME} {LIST_FORMATS_NAME}`

        - `file_conversor {COMMAND_NAME} {LIST_FORMATS_NAME} -f avi`
    """)
def list_formats(
    file_format: Annotated[str | None, FormatOption(FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)] = None,
):
    format_dict = FFmpegBackend.SUPPORTED_OUT_FORMATS
    if file_format:
        format_dict = {file_format: format_dict[file_format]}

    for fmt, fmt_info in format_dict.items():
        args, kwargs = fmt_info
        logger.info(f"[bold]{fmt}[/bold]")
        logger.info(f"  - {_('Audio codecs')}: {', '.join(kwargs['available_audio_codecs'])}")
        logger.info(f"  - {_('Video codecs')}: {', '.join(kwargs['available_video_codecs'])}")
    print()  # add a final newline


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]



# src/file_conversor/backend/gui/audio/info.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.audio_video import FFmpegBackend

from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.audio._dom_page import audio_index_nav_item, audio_info_nav_item

from file_conversor.utils.bulma_utils import InputFilesField
from file_conversor.utils.dominate_bulma import PageForm

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageAudioInfo():
    return PageForm(
        InputFilesField(
            *[f for f in FFmpegBackend.SUPPORTED_IN_AUDIO_FORMATS],
            description=_("Audio files"),
        ),
        api_endpoint=f"{url_for('api_audio_info')}",
        nav_items=[
            home_nav_item(),
            audio_index_nav_item(),
            audio_info_nav_item(active=True),
        ],
        _title=f"{_('Audio Info')} - File Conversor",
    )


def audio_info():
    return render_template_string(str(
        PageAudioInfo()
    ))


__all__ = ['audio_info']



# src/file_conversor/backend/gui/_api/status.py

from flask import json, render_template, request, url_for

# user-provided modules
from file_conversor.backend.gui.flask_api import FlaskApi

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def api_status():
    """API endpoint to get the application status."""
    params = FlaskApi.get_args()
    status_id = str(params.get('id', '-1'))
    return FlaskApi.status_response(status_id)



# src\file_conversor\backend\gui\_components\navbar.py

from typing import Any

from file_conversor.utils.dominate_utils import a, div, hr, img, nav, span


def NavbarDivider(_class: str = "", **kwargs):
    """
    Create a Navbar divider component.

    :param _class: Additional CSS classes to apply to the divider.
    """
    return hr(_class=f"navbar-divider {_class}", **kwargs)


def NavbarItem(*content, _href: str, _class: str = "", **kwargs):
    """
    Create a Navbar item component.

    :param content: The content to display inside the navbar item.
    :param _href: The URL the navbar item links to.
    :param _class: Additional CSS classes to apply to the navbar item.
    """
    with a(_class=f"navbar-item {_class}", _href=_href, **kwargs) as item:
        for c in content:
            if not c:
                continue
            item.add(c)
    return item


def NavbarDropdown(*items, label: str, _class: str = "", **kwargs):
    """
    Create a Navbar dropdown component.

    :param items: The items to include in the dropdown.
    :param label: The label for the dropdown toggle.    
    :param _class: Additional CSS classes to apply to the dropdown.
    """
    with div(_class=f"navbar-item has-dropdown {_class}", **kwargs) as dropdown:
        with a(_class="navbar-link"):
            dropdown.add(label)
        with div(_class="navbar-dropdown"):
            for item in items:
                if not item:
                    continue
                dropdown.add(item)
    return dropdown


def Navbar(
        title: Any = None,
        navbar_start: list[Any] | None = None,
        navbar_end: list[Any] | None = None,
        **kwargs
):
    """
    Create a Navbar component.

    :param title: The title or breadcrumb component to display in the navbar.
    :param navbar_start: Additional components to display on the left side of the navbar.
    :param navbar_end: Additional components to display on the right side of the navbar.
    """
    with nav(
        _class="navbar is-fixed-top is-light has-border has-shadow is-flex "
               "is-justify-content-space-between is-align-items-center",
        _role="navigation",
        _aria_label="main navigation",
        **{"x-data": "{ open: false }"},
        **kwargs,
    ) as component:

        # Navbar brand section (logo and burger)
        with div(_class="navbar-brand"):
            with a(_class="navbar-item", _href="/"):
                img(_height="140", _src="/icons/icon.png", _alt="")
                span("File Conversor", _class="is-size-6 has-text-weight-bold")

            # Mobile burger button
            with a(
                _role="button",
                _class="navbar-burger",
                _aria_label="menu",
                _aria_expanded="false",
                _data_target="navbarMenuContent",
                **{
                    "@click": "open = ! open",
                }
            ):
                span(_aria_hidden="true")
                span(_aria_hidden="true")
                span(_aria_hidden="true")
                span(_aria_hidden="true")

        # Navbar title + breadcrumbs
        with div(_class="navbar-title is-flex is-align-items-center") as title_container:
            if title:
                title_container.add(title)

        # Navbar menu (right-aligned)
        with div(
            _id="navbarMenuContent",
            **{":class": "open ? 'navbar-menu is-active' : 'navbar-menu'"}
        ):
            # Navbar start (left side)
            with div(_class="navbar-start") as navbar_start_container:
                navbar_start_container.add(*(navbar_start or []))

            # Navbar end (right side)
            with div(_class="navbar-end") as navbar_end_container:
                navbar_end_container.add(*(navbar_end or []))
    return component


__all__ = [
    "Navbar",
    "NavbarItem",
    "NavbarDropdown",
    "NavbarDivider",
]




# src\file_conversor\cli\multimedia\filter_cmd.py
import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend.image import PillowBackend
from file_conversor.cli.image._typer import FILTER_PANEL as RICH_HELP_PANEL
from file_conversor.cli.image._typer import COMMAND_NAME, FILTER_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import InputFilesArgument, OutputDirOption
from file_conversor.utils.validators import check_is_bool_or_none, check_path_exists, check_valid_options

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = PillowBackend.EXTERNAL_DEPENDENCIES


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # IMG2PDF commands
    for ext in PillowBackend.SUPPORTED_IN_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="blur",
                description="Blur",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{FILTER_NAME}" "%1" --filter blur"',
                icon=str(icons_folder_path / "blur.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


def execute_image_filter_cmd(
    input_files: List[Path],
    filters: List[str],
    output_dir: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    pillow_backend = PillowBackend(verbose=STATE['verbose'])

    def callback(input_file: Path, output_file: Path, progress_mgr: ProgressManager):
        logger.info(f"Processing '{output_file}' ... ")
        pillow_backend.filter(
            input_file=input_file,
            output_file=output_file,
            filters=filters,
        )
        progress_callback(progress_mgr.complete_step())

    cmd_mgr = CommandManager(input_files, output_dir=output_dir, overwrite=STATE["overwrite-output"])
    cmd_mgr.run(callback, out_stem="_filtered")

    logger.info(f"{_('Image filter')}: [green bold]{_('SUCCESS')}[/]")


@typer_cmd.command(
    name=FILTER_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Applies filter to an image file.')}
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {FILTER_NAME} input_file.jpg -od D:/Downloads --filter blur`

        - `file_conversor {COMMAND_NAME} {FILTER_NAME} input_file1.bmp -f sharpen`

        - `file_conversor {COMMAND_NAME} {FILTER_NAME} input_file.jpg -f sharpen -f 3d`        
    """)
def filter(
    input_files: Annotated[List[str], InputFilesArgument(PillowBackend)],

    filters: Annotated[List[str], typer.Option("--filter", "-f",
                                               help=f'{_("Filter to apply. Available filters:")} {", ".join(PillowBackend.PILLOW_FILTERS)}',
                                               callback=lambda x: [check_valid_options(opt, PillowBackend.PILLOW_FILTERS) for opt in x],
                                               )],

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    execute_image_filter_cmd(
        input_files=[Path(f) for f in input_files],
        filters=filters,
        output_dir=output_dir,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_image_filter_cmd",
]




# src\file_conversor\cli\config\set_cmd.py

from pathlib import Path
import typer

from typing import Annotated

from rich import print

# user-provided modules
import file_conversor.cli.config.show_cmd as show_cmd

from file_conversor.cli.config._typer import COMMAND_NAME, SET_NAME

from file_conversor.backend import Img2PDFBackend, PillowBackend, FFmpegBackend, GhostscriptBackend
from file_conversor.config import Configuration, State, Log, locale, get_translation, AVAILABLE_LANGUAGES

from file_conversor.utils.typer_utils import VideoEncodingSpeedOption, VideoQualityOption
from file_conversor.utils.validators import check_dir_exists, check_is_bool_or_none, check_positive_integer, check_valid_options, check_ip_format

# app configuration
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

# create command
typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = set()


@typer_cmd.command(
    name=SET_NAME,
    help=f"""
        {_('Configure the default options for the app.')}
    """,
    epilog=f"""
        **{_('Examples')}:** 

        - `file_conversor configure --video-bitrate 5000`

        - `file_conversor configure --audio-bitrate 128`
    """,
)
def set(
    cache_enabled: Annotated[bool, typer.Option("--cache-enabled", "-ce",
                                                help=_("Enable or disable HTTP cache."),
                                                callback=check_is_bool_or_none,
                                                is_flag=True,
                                                )] = CONFIG["cache-enabled"],


    cache_expire_after: Annotated[int, typer.Option("--cache-expire-after", "-cea",
                                                    help=_("Set HTTP cache expiration time in seconds."),
                                                    min=1,
                                                    )] = CONFIG["cache-expire-after"],

    language: Annotated[str, typer.Option("--language", "-l",
                                          help=f'{_("Set preferred language for app (if available). Available languages:")} {", ".join(sorted(AVAILABLE_LANGUAGES))}. {_("Defaults to system preffered language or 'en_US' (English - United States)")}.',
                                          callback=lambda x: check_valid_options(x, AVAILABLE_LANGUAGES),
                                          )] = locale.normalize_lang_code(CONFIG["language"]) or locale.get_default_language(),

    install_deps: Annotated[str | None, typer.Option("--install-deps", "-install",
                                                     help=_("Install missing external dependencies action. 'True' for auto install. 'False' to not install missing dependencies. 'None' to ask user for action."),
                                                     callback=check_is_bool_or_none,
                                                     )] = CONFIG["install-deps"],

    port: Annotated[int, typer.Option("--port", "-p",
                                      help=f'{_("Set preferred listen port for app (if available). Ports below 1024 require root privileges. Defaults to 5000.")}.',
                                      min=1, max=65535,
                                      )] = CONFIG["port"],

    audio_bitrate: Annotated[int, typer.Option("--audio-bitrate", "-ab",
                                               help=f"{_("Audio bitrate in kbps.")} {_('If 0, let FFmpeg decide best bitrate.')}",
                                               callback=lambda x: check_positive_integer(x, allow_zero=True),
                                               )] = CONFIG["audio-bitrate"],

    video_bitrate: Annotated[int, typer.Option("--video-bitrate", "-vb",
                                               help=f"{_("Video bitrate in kbps.")} {_('If 0, let FFmpeg decide best bitrate.')}",
                                               callback=lambda x: check_positive_integer(x, allow_zero=True),
                                               )] = CONFIG["video-bitrate"],

    video_format: Annotated[str, typer.Option("--video-format", "-vf",
                                              help=f"{_("Video format.")} {_('Available formats:')} {", ".join(FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)}",
                                              callback=lambda x: check_valid_options(x, FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS),
                                              )] = CONFIG["video-format"],

    video_encoding_speed: Annotated[str, VideoEncodingSpeedOption(FFmpegBackend.ENCODING_SPEEDS)] = CONFIG["video-encoding-speed"],

    video_quality: Annotated[str, VideoQualityOption(FFmpegBackend.QUALITY_PRESETS)] = CONFIG["video-quality"],

    image_quality: Annotated[int, typer.Option("--image-quality", "-iq",
                                               help=_("Image quality (for ``image convert`` command). Valid values are between 1-100."),
                                               min=1, max=100,
                                               )] = CONFIG["image-quality"],

    image_dpi: Annotated[int, typer.Option("--image-dpi", "-id",
                                           help=_("Image quality in dots per inch (DPI) (for ``image to_pdf`` command). Valid values are between 40-3600."),
                                           min=40, max=3600,
                                           )] = CONFIG["image-dpi"],

    image_fit: Annotated[str, typer.Option("--image-fit", "-if",
                                           help=f'{_("Image fit (for ``image to_pdf`` command). Valid only if ``--page-size`` is defined. Valid values are")} {", ".join(Img2PDFBackend.FIT_MODES)}',
                                           callback=lambda x: check_valid_options(x.lower(), ['into', 'fill']),
                                           )] = CONFIG["image-fit"],

    image_page_size: Annotated[str | None, typer.Option("--image-page-size", "-ip",
                                                        help=f'{_("Page size (for ``image to_pdf`` command). Format (width, height). Other valid values are:")} {", ".join(Img2PDFBackend.PAGE_LAYOUT)}',
                                                        callback=lambda x: check_valid_options(x.lower() if x else None, Img2PDFBackend.PAGE_LAYOUT),
                                                        )] = CONFIG["image-page-size"],

    image_resampling: Annotated[str, typer.Option("--image-resampling", "-ir",
                                                  help=f'{_("Resampling algorithm. Valid values are")} {", ".join(PillowBackend.RESAMPLING_OPTIONS)}. {_("Defaults to")} {CONFIG["image-resampling"]}',
                                                  callback=lambda x: check_valid_options(x, PillowBackend.RESAMPLING_OPTIONS),
                                                  )] = CONFIG["image-resampling"],

    pdf_compression: Annotated[str, typer.Option("--pdf-compression", "-pc",
                                                 help=f"{_('Compression level (high compression = low quality). Valid values are')} {', '.join(GhostscriptBackend.Compression.get_dict())}. {_('Defaults to')} {CONFIG["pdf-compression"]}.",
                                                 callback=lambda x: check_valid_options(x, GhostscriptBackend.Compression.get_dict()),
                                                 )] = CONFIG["pdf-compression"],

    gui_zoom: Annotated[int, typer.Option("--gui-zoom", "-gz",
                                          help=_("GUI zoom level. Valid values are >= 1 (100 = normal size, 150 = 1.5x size, etc)."),
                                          min=1,
                                          )] = CONFIG["gui-zoom"],

    gui_output_dir: Annotated[Path, typer.Option("--gui-output-dir", "-god",
                                                 help=f"{_('GUI output directory')}. {_('Defaults to')} {CONFIG["gui-output-dir"]}.",
                                                 callback=lambda x: check_dir_exists(x),
                                                 )] = CONFIG["gui-output-dir"],
):
    # update the configuration dictionary
    CONFIG.update({
        "cache-enabled": cache_enabled,
        "cache-expire-after": cache_expire_after,
        "port": port,
        "language": language,
        "install-deps": None if install_deps == "None" or install_deps is None else bool(install_deps),
        "audio-bitrate": audio_bitrate,
        "video-bitrate": video_bitrate,
        "video-format": video_format,
        "video-encoding-speed": video_encoding_speed,
        "video-quality": video_quality,
        "image-quality": image_quality,
        "image-dpi": image_dpi,
        "image-fit": image_fit,
        "image-page-size": image_page_size,
        "image-resampling": image_resampling,
        "pdf-compression": pdf_compression,
        "gui-zoom": gui_zoom,
        "gui-output-dir": gui_output_dir,
    })
    CONFIG.save()
    show_cmd.show()
    logger.info(f"{_('Configuration file')} {_('updated')}.")


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]



# src/file_conversor/backend/gui/_components/visibility.py

class Visibility:
    IS_BLOCK = "is-block"
    IS_INLINE = "is-inline"
    IS_INLINE_BLOCK = "is-inline-block"
    IS_FLEX = "is-flex"
    IS_INLINE_FLEX = "is-inline-flex"
    IS_INVISIBLE = "is-invisible"
    IS_HIDDEN = "is-hidden"
    IS_SR_ONLY = "is-sr-only"


class IsFlexOnly:
    MOBILE = "is-flex-mobile"
    TABLET = "is-flex-tablet-only"
    DESKTOP = "is-flex-desktop-only"
    WIDESCREEN = "is-flex-widescreen-only"


class IsFlexUpTo:
    TOUCH = "is-flex-touch"
    TABLET = "is-flex-tablet"
    DESKTOP = "is-flex-desktop"
    WIDESCREEN = "is-flex-widescreen"
    FULLHD = "is-flex-fullhd"


class IsHiddenOnly:
    MOBILE = "is-hidden-mobile"
    TABLET = "is-hidden-tablet-only"
    DESKTOP = "is-hidden-desktop-only"
    WIDESCREEN = "is-hidden-widescreen-only"


class IsHiddenUpTo:
    TOUCH = "is-hidden-touch"
    TABLET = "is-hidden-tablet"
    DESKTOP = "is-hidden-desktop"
    WIDESCREEN = "is-hidden-widescreen"
    FULLHD = "is-hidden-fullhd"


__all__ = [
    'Visibility',
    'IsFlexOnly',
    'IsFlexUpTo',
    'IsHiddenOnly',
    'IsHiddenUpTo',
]



"""
Module for LibreOffice backend (calc, writer, etc)
"""

from file_conversor.backend.office.calc_backend import *
from file_conversor.backend.office.impress_backend import *
from file_conversor.backend.office.writer_backend import *



# src\file_conversor\utils\bulma\form.py

from typing import Any

from file_conversor.utils.dominate_bulma.form_field import FormField
from file_conversor.utils.dominate_utils import div, option, select


def Select(
        *options: tuple[str, str],
        placeholder: str = "",
        _class: str = "",
        _class_container: str = "",
        kwargs_container: dict[str, Any] | None = None,
        **kwargs,
):
    """
    Create a select dropdown.

    :param options: A list of tuples where each tuple contains (value, display_text).
    :param placeholder: Placeholder text for the select dropdown.
    :param _class: Additional CSS classes for the select element.
    :param _class_container: Additional CSS classes for the select container.
    """
    with div(_class=f"select {_class_container}", **(kwargs_container if kwargs_container else {})) as select_el:
        with select(_class=f"is-full-width {_class}", **kwargs,):
            if placeholder and not options:
                option(placeholder, _disabled=True, _selected=True, _hidden=True)
            for value, display_text in options:
                option(display_text, _value=value)
    return select_el


def FormFieldSelect(
    *options: tuple[str, str],
    help: str,
    _name: str,
    label_text: str = "",
    current_value: str | None = None,
    x_data: str = "",
    x_init: str = "",
    **kwargs,
):
    """
    Create a form field with a label and a select element.

    :param options: A list of tuples where each tuple contains (value, display_text).
    :param current_value: The current selected value.
    :param label_text: The text for the label.
    :param help: Optional help text.
    :param _name: The name attribute for the select element.
    :param x_data: Additional Alpine.js x-data properties.

    :return: The FormField element.
    """
    field = FormField(
        Select(
            *options,
            _class="is-flex is-flex-grow-1",
            _name=_name,
            _title=help,
            kwargs_container={
                ':class': """{
                    'is-danger': !isValid,
                    'is-success': isValid,
                }""",
            },
            **{'x-model': 'value'},
            **kwargs,
        ),
        current_value=current_value,
        _class_control="is-flex is-flex-direction-column is-flex-grow-1",
        label_text=label_text,
        help=help,
        x_data=x_data,
        x_init=x_init,
    )
    return field


__all__ = [
    "Select",
    "FormFieldSelect",
]



# src/file_conversor/backend/gui/config/_tab_pdf.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.utils.bulma_utils import PDFCompressionField

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation, AVAILABLE_LANGUAGES

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def TabConfigPDF() -> tuple | list:
    return (
        PDFCompressionField(),
    )


__all__ = ['TabConfigPDF']



# src/file_conversor/backend/gui/_api/audio/__init__.py

from file_conversor.backend.gui.flask_route import FlaskRoute

from file_conversor.backend.gui._api.audio.check import api_audio_check
from file_conversor.backend.gui._api.audio.convert import api_audio_convert
from file_conversor.backend.gui._api.audio.info import api_audio_info


def routes():
    return [
        FlaskRoute(
            rule="/api/audio/check",
            handler=api_audio_check,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/audio/convert",
            handler=api_audio_convert,
            methods=["POST"],
        ),
        FlaskRoute(
            rule="/api/audio/info",
            handler=api_audio_info,
            methods=["POST"],
        ),
    ]


__all__ = ['routes']



# src\file_conversor\cli\ppt\_info.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

# command
COMMAND_NAME = "ppt"

# SUBCOMMANDS
CONVERT_NAME = "convert"

__all__ = [
    "COMMAND_NAME",

    "CONVERT_NAME",
]



# src/file_conversor/backend/gui/image/convert.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.image import PillowBackend

from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.image._dom_page import image_convert_nav_item, image_index_nav_item

from file_conversor.utils.bulma_utils import FileFormatField, InputFilesField, OutputDirField, ImageQualityField
from file_conversor.utils.dominate_bulma import PageForm

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def PageImageConvert():
    return PageForm(
        InputFilesField(
            *[f for f in PillowBackend.SUPPORTED_IN_FORMATS],
            description=_("Image files"),
        ),
        FileFormatField(*[
            (q, q.upper())
            for q in PillowBackend.SUPPORTED_OUT_FORMATS
        ], current_value='jpg'),
        ImageQualityField(),
        OutputDirField(),
        api_endpoint=f"{url_for('api_image_convert')}",
        nav_items=[
            home_nav_item(),
            image_index_nav_item(),
            image_convert_nav_item(active=True),
        ],
        _title=f"{_('Image Convert')} - File Conversor",
    )


def image_convert():
    return render_template_string(str(
        PageImageConvert()
    ))



# src\file_conversor\backend\http_backend.py

"""
This module provides functionalities for handling repositories using HTTP.
"""

import requests
import requests_cache

from pathlib import Path
from typing import Any, Callable, Iterable

# user-provided imports
from file_conversor.backend.abstract_backend import AbstractBackend

from file_conversor.config import Configuration, State, Environment, Log, AbstractSingletonThreadSafe, get_translation

from file_conversor.utils.validators import check_file_format

_ = get_translation()
LOG = Log.get_instance()
CONFIG = Configuration.get_instance()

logger = LOG.getLogger(__name__)


class NetworkError(RuntimeError):
    """Base exception for network-related errors in HttpBackend."""


class HttpBackend(AbstractBackend, AbstractSingletonThreadSafe):
    """
    HttpBackend is a class that provides an interface for handling HTTP requests.
    """

    SUPPORTED_IN_FORMATS = {}

    SUPPORTED_OUT_FORMATS = {}

    EXTERNAL_DEPENDENCIES = set()

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the backend.

        :param verbose: Verbose logging. Defaults to False.    
        :param cache_enabled: Enable caching. Defaults to True.  
        :param cache_name: Name of the cache file. Defaults to ".http_cache".  
        :param cache_expire_after: Cache expiration time in seconds. Defaults to 1 day
        """
        super().__init__()
        self._verbose = verbose
        self._cache_file = Environment.get_data_folder() / ".http_cache.sqlite"

        # Creates a persistent SQLite cache file: http_cache.sqlite
        self._cached_session = requests_cache.CachedSession(
            cache_name=str(self._cache_file.with_suffix("").resolve()),
            backend="sqlite",
            expire_after=int(CONFIG["cache-expire-after"]),
            allowable_codes=(200,),  # ONLY cache successful HTTP 200 responses
            stale_if_error=True,     # Use stale cache if network error occurs
            old_data_on_error=True,  # Use old cache data if http 500 error occurs
            fast_save=True,          # Faster cache saving
        )
        self._plain_session = requests.Session()

    def clear_cache(self):
        """Clear the HTTP cache."""
        requests_cache.clear()
        logger.warning(_("HTTP cache cleared."))

    def get_json(
        self,
        url: str,
        cache_enabled: bool = True,
        timeout: tuple[int, int] | None = (5, 10),
        **kwargs,
    ) -> Any:
        """Get JSON data from a URL.

        :param url: The URL to fetch the JSON data from.        
        :param cache_enabled: Whether to use the cache. Defaults to True.
        :param timeout: A tuple specifying the connection and read timeout in seconds. Defaults to (5,30).
        :param kwargs: Additional arguments to pass to the requests.get() method.
        :return: The JSON data.

        :raises NetworkError: if the request fails or the response is not JSON.
        """
        session = self._cached_session if cache_enabled else self._plain_session
        try:
            response = session.get(url, timeout=timeout, **kwargs)
            if not response.ok:
                raise NetworkError(f"Response code: {response.status_code} - {response.text}")
            return response.json()
        except Exception as e:
            raise NetworkError(f"{_('Error during JSON fetch from url')} '{url}': {repr(e)}")

    def download(
        self,
        url: str,
        dest_file: str | Path,
        cache_enabled: bool = False,
        timeout: tuple[int, int] | None = (5, 600),
        progress_callback: Callable[[float], Any] | None = None,
        **kwargs,
    ):
        """
        Download a file from a URL.

        :param url: The URL to download the file from.
        :param dest_file: The destination file path.
        :param cache_enabled: Whether to use the cache. Defaults to False.
        :param timeout: A tuple specifying the connection and read timeout in seconds. Defaults to (5,600).
        :param progress_callback: A callback function to report download progress (0-100). Defaults to None.
        :param kwargs: Additional arguments to pass to the requests.get() method.

        :raises NetworkError: if the download fails.
        """
        dest_file = Path(dest_file).resolve()
        dest_file.parent.mkdir(parents=True, exist_ok=True)
        downloaded_bytes = 0.0

        session = self._cached_session if cache_enabled else self._plain_session
        try:
            with session.get(url, timeout=timeout, stream=True, **kwargs) as response:
                if not response.ok:
                    raise NetworkError(f"{_('Response code')}: {response.status_code} - {response.text}")
                total_size = float(response.headers.get("content-length", 0))
                with dest_file.open("wb") as f:
                    for data in response.iter_content(chunk_size=1024):
                        f.write(data)
                        downloaded_bytes += len(data)
                        if progress_callback:
                            progress_callback(downloaded_bytes / total_size * 100)
            if not dest_file.exists():
                raise FileNotFoundError(f"'{dest_file}'")
        except Exception as e:
            raise NetworkError(f"{_('Network error during download')}: {repr(e)}")


__all__ = [
    "NetworkError",
    "HttpBackend",
]



# src\file_conversor\utils\bulma_utils\image.py

from typing import Any

# user-provided modules
from file_conversor.backend.image import PillowBackend, Img2PDFBackend

from file_conversor.utils.dominate_bulma.form_checkbox import FormFieldCheckbox
from file_conversor.utils.dominate_bulma.form_field import FormFieldHorizontal
from file_conversor.utils.dominate_bulma.form_input import FormFieldInput
from file_conversor.utils.dominate_bulma.form_select import FormFieldSelect

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


def ImageQualityField():
    """Create a form field for image quality adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) >= 0 && Number.parseInt(value) <= 100",
            current_value=CONFIG["image-quality"],
            _name="image-quality",
            _type="number",
            step="10",
            help=_("Adjust image quality level. Value between 0 (lowest) to 100 (highest)."),
        ),
        label_text=_("Image Quality (%)"),
    )


def ImageRadiusField():
    """Create a form field for image radius adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) >= 1",
            current_value='3',
            _name="radius",
            _type="number",
            step="1",
            help=_("Box radius (in pixels). Must be a positive integer."),
        ),
        label_text=_("Box Radius (px)"),
    )


def ImageAntialiasAlgorithmField():
    """Create a form field for selecting antialias algorithm."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                (k, k.upper())
                for k in PillowBackend.AntialiasAlgorithm.get_dict()
            ],
            current_value='median',
            _name="algorithm",
            help=_("Select the antialias algorithm to use."),
        ),
        label_text=_("Algorithm"),
    )


def ImageFilterField():
    """Create a form field for image filter options."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                (k, k.upper())
                for k in PillowBackend.PILLOW_FILTERS
            ],
            _name="filter",
            help=_("Select the filter to apply to the image."),
        ),
        label_text=_("Filter"),
    )


def ImageDPIField():
    """Create a form field for DPI selection."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) > 0",
            current_value=str(CONFIG["image-dpi"]),
            _name="image-dpi",
            _type="number",
            step="100",
            help=_("Dots per inch (DPI) for rendering the image. Must be a positive integer."),
        ),
        label_text=_("DPI"),
    )


def ImageRotationField():
    """Create a form field for image rotation."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) >= -360 && Number.parseInt(value) <= 360",
            current_value='0',
            _name="image-rotation",
            _type="number",
            step="90",
            help=_("Rotation in degrees. Valid values are between -360 (anti-clockwise) and 360 (clockwise) rotation."),
        ),
        label_text=_("Rotation (deg)"),
    )


def ImageScaleField():
    """Create a form field for image scale adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseFloat(value) > 0",
            current_value='1.0',
            _name="image-scale",
            _type="number",
            step="0.1",
            help=_("Scale factor for resizing the image. Must be a positive number."),
        ),
        label_text=_("Scale Factor"),
    )


def ImageResampleAlgorithmField():
    """Create a form field for selecting resample algorithm."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                (k, k.upper())
                for k in PillowBackend.RESAMPLING_OPTIONS
            ],
            current_value=CONFIG["image-resampling"],
            _name="image-resampling",
            help=_("Select the resampling algorithm to use when resizing images."),
        ),
        label_text=_("Resampling Algorithm"),
    )


def ImageFitField():
    """Create a form field for image fit option."""
    return FormFieldHorizontal(
        FormFieldSelect(
            *[
                (k, k.upper())
                for k in Img2PDFBackend().FIT_MODES
            ],
            current_value=CONFIG["image-fit"],
            _name="image-fit",
            help=_("Select how to fit the image into the page when converting to PDF. Valid only if page size is defined."),
        ),
        label_text=_("Image Fit"),
    )


def ImagePageSizeField():
    """Create a form field for image page size option."""
    return FormFieldHorizontal(
        FormFieldSelect(
            ('null', _('Not defined')),
            *[
                (k, k.upper())
                for k in Img2PDFBackend().PAGE_LAYOUT
            ],
            current_value=CONFIG["image-page-size"],
            _name="image-page-size",
            help=_("Select the page size for the output PDF. If not defined, PDF size will match image size."),
        ),
        label_text=_("Page Size"),
    )


def ImageSetMetadataField():
    """Create a form field for setting image metadata option."""
    return FormFieldCheckbox(
        current_value="off",
        _name="image-set-metadata",
        help=_("Include image metadata (EXIF, IPTC, etc) in the output file."),
        label_text=_("Set Metadata"),
    )


def ImageUnsharpStrengthField():
    """Create a form field for image unsharp strength adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) >= 1",
            current_value='130',
            _name="image-unsharp-strength",
            _type="number",
            step="10",
            help=_("Unsharp strength, in percent. Must be a positive integer."),
        ),
        label_text=_("Strength (%)"),
    )


def ImageUnsharpThresholdField():
    """Create a form field for image unsharp threshold adjustment."""
    return FormFieldHorizontal(
        FormFieldInput(
            validation_expr="Number.parseInt(value) >= 1",
            current_value='4',
            _name="image-unsharp-threshold",
            _type="number",
            step="1",
            help=_("Threshold controls the minimum brightness change that will be sharpened. Must be a positive integer."),
        ),
        label_text=_("Threshold"),
    )


__all__ = [
    "ImageQualityField",
    "ImageRadiusField",
    "ImageAntialiasAlgorithmField",
    "ImageFilterField",
    "ImageDPIField",
    "ImageRotationField",
    "ImageScaleField",
    "ImageResampleAlgorithmField",
    "ImageFitField",
    "ImagePageSizeField",
    "ImageSetMetadataField",
    "ImageUnsharpStrengthField",
    "ImageUnsharpThresholdField",
]



# src/file_conversor/backend/gui/_api/video/rotate.py

from pathlib import Path
from typing import Any

# user-provided modules
from file_conversor.cli.video._ffmpeg_cmd_helper import FFmpegCmdHelper, EXTERNAL_DEPENDENCIES

from file_conversor.backend.gui.flask_api import FlaskApi
from file_conversor.backend.gui.flask_api_status import FlaskApiStatus

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)


def _api_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """Thread to handle video rotation."""
    logger.debug(f"Video rotate thread received: {params}")

    input_files = [Path(i) for i in params['input-files']]
    output_dir = Path(params['output-dir'])

    file_format = params['file-format']

    audio_bitrate = params['audio-bitrate']
    video_bitrate = params['video-bitrate']

    video_encoding_speed = params['video-encoding-speed']
    video_quality = params['video-quality']

    rotation = params['rotation'] or None

    logger.info(f"[bold]{_('Rotating video files')}[/]...")

    ffmpeg_cmd_helper = FFmpegCmdHelper(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
        overwrite_output=STATE["overwrite-output"],
    )

    # Set arguments for FFmpeg command helper
    ffmpeg_cmd_helper.set_input(input_files)
    ffmpeg_cmd_helper.set_output(file_format=file_format, out_stem="_rotated", output_dir=output_dir)

    ffmpeg_cmd_helper.set_video_settings(encoding_speed=video_encoding_speed, quality=video_quality)
    ffmpeg_cmd_helper.set_bitrate(audio_bitrate=audio_bitrate, video_bitrate=video_bitrate)

    ffmpeg_cmd_helper.set_rotation_filter(rotation)

    ffmpeg_cmd_helper.execute(
        progress_callback=lambda p, pm: status.set_progress(pm.update_progress(p)),
    )

    logger.debug(f"{status}")


def api_video_rotate():
    """API endpoint to rotate video files."""
    logger.info(f"[bold]{_('Video rotate requested via API.')}[/]")
    return FlaskApi.execute_response(_api_thread)


__all__ = [
    "api_video_rotate",
    "EXTERNAL_DEPENDENCIES",
]



# src\file_conversor\cli\vidio\_typer.py

# user-provided modules
from file_conversor.config import get_translation

_ = get_translation()

TRANSFORMATION_PANEL = _("Transformations")
OTHERS_PANEL = _("Other commands")

# command
COMMAND_NAME = "video"

# SUBCOMMANDS
CHECK_NAME = "check"
COMPRESS_NAME = "compress"
CONVERT_NAME = "convert"
ENHANCE_NAME = "enhance"
EXECUTE_NAME = "execute"
INFO_NAME = "info"
LIST_FORMATS_NAME = "list-formats"
MIRROR_NAME = "mirror"
RESIZE_NAME = "resize"
ROTATE_NAME = "rotate"

__all__ = [
    "COMMAND_NAME",

    "CHECK_NAME",
    "COMPRESS_NAME",
    "CONVERT_NAME",
    "ENHANCE_NAME",
    "EXECUTE_NAME",
    "INFO_NAME",
    "LIST_FORMATS_NAME",
    "MIRROR_NAME",
    "RESIZE_NAME",
    "ROTATE_NAME",
]




# src\file_conversor\cli\hash\create_cmd.py

import typer

from pathlib import Path
from typing import Annotated, Any, Callable, List

from rich import print

# user-provided modules
from file_conversor.backend import HashBackend

from file_conversor.cli.hash._typer import COMMAND_NAME, CREATE_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import FormatOption, InputFilesArgument, OutputDirOption
from file_conversor.utils.validators import check_path_exists

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()

EXTERNAL_DEPENDENCIES = HashBackend.EXTERNAL_DEPENDENCIES


def execute_hash_create_cmd(
    input_files: List[Path],
    output_file: Path,
    progress_callback: Callable[[float], Any] = lambda p: p,
):
    if not STATE["overwrite-output"]:
        check_path_exists(output_file, exists=False)

    hash_backend = HashBackend(
        verbose=STATE["verbose"],
    )
    with ProgressManager() as progress_mgr:
        hash_backend.generate(
            input_files=input_files,
            output_file=output_file,
            progress_callback=lambda p: progress_callback(progress_mgr.update_progress(p)),
        )
        progress_callback(progress_mgr.complete_step())

    logger.info(f"{_('Hash creation')}: [bold green]{_('SUCCESS')}[/].")


@typer_cmd.command(
    name=CREATE_NAME,
    help=f"""
        {_('Creates a hash file (.sha256, .sha1, etc).')}        
    """,
    epilog=f"""
**{_('Examples')}:** 

- `file_conversor {COMMAND_NAME} {CREATE_NAME} file1.jpg file2.pdf file3.exe -f sha256` 

- `file_conversor {COMMAND_NAME} {CREATE_NAME} file1.jpg file2.pdf -f sha1 -od D:/Downloads` 
""")
def create(
    input_files: Annotated[List[Path], InputFilesArgument()],
    format: Annotated[str, FormatOption(HashBackend)],
    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    output_file = output_dir / f"CHECKSUM.{format}"
    execute_hash_create_cmd(
        input_files=input_files,
        output_file=output_file,
    )


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
    "execute_hash_create_cmd",
]



# src/file_conversor/backend/gui/image/index.py

from flask import render_template, render_template_string, url_for

# user-provided modules
from file_conversor.backend.gui._dom_page import home_nav_item
from file_conversor.backend.gui.image._dom_page import image_index_nav_item

from file_conversor.utils.dominate_bulma import PageCardGrid

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()


def image_index():
    tools = [
        {

            'image': url_for('icons', filename='convert.ico'),
            'title': _("Convert files"),
            'subtitle': _("Convert a image file to a different format."),

            'url': url_for('image_convert'),
        },
        {

            'image': url_for('icons', filename='svg.ico'),
            'title': _("Render vector"),
            'subtitle': _("Render an image vector file into a different format."),

            'url': url_for('image_render'),
        },
        {

            'image': url_for('icons', filename='pdf.ico'),
            'title': _("To PDF"),
            'subtitle': _("Convert a images to one PDF file, one image per page."),

            'url': url_for('image_to_pdf'),
        },
        {

            'image': url_for('icons', filename='compress.ico'),
            'title': _("Compress file"),
            'subtitle': _("Compress an image file (requires external libraries)."),

            'url': url_for('image_compress'),
        },
        {

            'image': url_for('icons', filename='left_right.ico'),
            'title': _("Mirror / Flip"),
            'subtitle': _("Mirror an image file (vertically or horizontally)."),

            'url': url_for('image_mirror'),
        },
        {

            'image': url_for('icons', filename='rotate_right.ico'),
            'title': _("Rotate"),
            'subtitle': _("Rotate a image file (clockwise or anti-clockwise)."),

            'url': url_for('image_rotate'),
        },
        {

            'image': url_for('icons', filename='resize.ico'),
            'title': _("Resize"),
            'subtitle': _("Resize an image file resolution (upscale or downscale)."),

            'url': url_for('image_resize'),
        },
        {

            'image': url_for('icons', filename='diagonal_line.ico'),
            'title': _("Antialias"),
            'subtitle': _("Applies antialias filter to an image file."),

            'url': url_for('image_antialias'),
        },
        {

            'image': url_for('icons', filename='blur.ico'),
            'title': _("Blur"),
            'subtitle': _("Applies gaussian blur to an image file."),

            'url': url_for('image_blur'),
        },
        {

            'image': url_for('icons', filename='color.ico'),
            'title': _("Enhance"),
            'subtitle': _("Enhance image color, brightness, contrast, or sharpness."),

            'url': url_for('image_enhance'),
        },
        {

            'image': url_for('icons', filename='filter.ico'),
            'title': _("Filter"),
            'subtitle': _("Applies multiples filters to an image file."),

            'url': url_for('image_filter'),
        },
        {

            'image': url_for('icons', filename='sharpener.ico'),
            'title': _("Sharpen"),
            'subtitle': _("Applies unsharp mask to an image file."),

            'url': url_for('image_unsharp'),
        },
        {

            'image': url_for('icons', filename='info.ico'),
            'title': _("Get info"),
            'subtitle': _("Get EXIF information about a image file."),

            'url': url_for('image_info'),
        },
    ]

    return render_template_string(str(
        PageCardGrid(
            *tools,
            nav_items=[
                home_nav_item(),
                image_index_nav_item(active=True),
            ],
            _title=f"{_('Image Tools')} - File Conversor",
        ))
    )



# src\file_conversor\utils\bulma\form.py

from typing import Any

from file_conversor.utils.dominate_bulma.font_awesome_icon import FontAwesomeIcon
from file_conversor.utils.dominate_utils import form


def Form(
        *form_fields,
        _class: str = "",
        **kwargs,
):
    """
    Create a form element.

    :param form_fields: The form field elements (e.g., FormField).
    :param _class: Additional CSS classes for the form.
    :param kwargs: Additional attributes for the form element.
    """
    with form(_class=f"form {_class}", **kwargs) as form_elem:
        for form_field in form_fields:
            if not form_field:
                continue
            form_elem.add(form_field)
    return form_elem


__all__ = [
    "Form",
]



# src\file_conversor\cli\image\__init__.py

import typer

# user-provided modules
from file_conversor.config.locale import get_translation

from file_conversor.cli.image._typer import COMMAND_NAME

from file_conversor.cli.image.antialias_cmd import typer_cmd as antialias_cmd
from file_conversor.cli.image.blur_cmd import typer_cmd as blur_cmd
from file_conversor.cli.image.compress_cmd import typer_cmd as compress_cmd
from file_conversor.cli.image.convert_cmd import typer_cmd as convert_cmd
from file_conversor.cli.image.enhance_cmd import typer_cmd as enhance_cmd
from file_conversor.cli.image.filter_cmd import typer_cmd as filter_cmd
from file_conversor.cli.image.info_cmd import typer_cmd as info_cmd
from file_conversor.cli.image.mirror_cmd import typer_cmd as mirror_cmd
from file_conversor.cli.image.render_cmd import typer_cmd as render_cmd
from file_conversor.cli.image.resize_cmd import typer_cmd as resize_cmd
from file_conversor.cli.image.rotate_cmd import typer_cmd as rotate_cmd
from file_conversor.cli.image.to_pdf_cmd import typer_cmd as to_pdf_cmd
from file_conversor.cli.image.unsharp_cmd import typer_cmd as unsharp_cmd

_ = get_translation()

image_cmd = typer.Typer(
    name=COMMAND_NAME,
    help=_("Image file manipulation"),
)
# CONVERSION_PANEL
image_cmd.add_typer(convert_cmd)
image_cmd.add_typer(render_cmd)
image_cmd.add_typer(to_pdf_cmd)

# TRANSFORMATION_PANEL
image_cmd.add_typer(compress_cmd)
image_cmd.add_typer(mirror_cmd)
image_cmd.add_typer(rotate_cmd)
image_cmd.add_typer(resize_cmd)

# FILTER_PANEL
image_cmd.add_typer(antialias_cmd)
image_cmd.add_typer(blur_cmd)
image_cmd.add_typer(enhance_cmd)
image_cmd.add_typer(filter_cmd)
image_cmd.add_typer(unsharp_cmd)

# OTHERS_PANEL
image_cmd.add_typer(info_cmd)

__all__ = [
    "image_cmd",
]



# src\file_conversor\backend\pypdf_backend.py

"""
This module provides functionalities for handling PDF files using ``pypdf`` backend.
"""

from pypdf import PdfReader, PdfWriter
from pypdf.constants import UserAccessPermissions

from enum import Enum
from pathlib import Path

from typing import Any, Callable, Iterable, Sequence

# user-provided imports
from file_conversor.config import Log

from file_conversor.utils.formatters import normalize_degree

from file_conversor.backend.abstract_backend import AbstractBackend

LOG = Log.get_instance()

logger = LOG.getLogger(__name__)


class PyPDFBackend(AbstractBackend):
    """
    A class that provides an interface for handling PDF files using ``pypdf``.
    """

    SUPPORTED_IN_FORMATS = {
        "pdf": {},
    }
    SUPPORTED_OUT_FORMATS = {
        "pdf": {},
    }
    EXTERNAL_DEPENDENCIES: set[str] = set()

    class EncryptionPermission(Enum):
        NONE = 'none'
        """ No permissions granted to user (read-only) """

        ANNOTATE = 'annotate'
        """ User can add/modify annotations (comments, highlight text, etc) and interactive forms. """

        FILL_FORMS = 'fill_forms'
        """ User can fill interactive form fields. """

        MODIFY = 'modify'
        """ User can modify the document (except pages). """

        MODIFY_PAGES = 'modify_pages'
        """ User can modify the document's pages (insert, delete, rotate, etc). """

        COPY = 'copy'
        """ User can copy text and graphics from the document. """

        ACCESSIBILITY = 'accessibility'
        """ User can extract text and graphics for accessibility purposes. """

        PRINT_LQ = 'print_lq'
        """ User can print the document in low quality. """

        PRINT_HQ = 'print_hq'
        """ User can print the document in high quality. """

        ALL = 'all'
        """ All permissions granted to user. """

        def get_value(self):
            """Get permission value."""
            return {
                'none': UserAccessPermissions(0),
                'annotate': UserAccessPermissions.ADD_OR_MODIFY,
                'fill_forms': UserAccessPermissions.FILL_FORM_FIELDS,
                'modify': UserAccessPermissions.MODIFY,
                'modify_pages': UserAccessPermissions.ASSEMBLE_DOC,
                'copy': UserAccessPermissions.EXTRACT,
                'accessibility': UserAccessPermissions.EXTRACT_TEXT_AND_GRAPHICS,
                'print_lq': UserAccessPermissions.PRINT,
                'print_hq': UserAccessPermissions.PRINT_TO_REPRESENTATION,
                'all': UserAccessPermissions.all(),
            }[self.value]

        @classmethod
        def get_dict(cls) -> dict[str, 'PyPDFBackend.EncryptionPermission']:
            """Get dictionary of encryption algorithms."""
            return dict(cls.__members__)

        @classmethod
        def from_str(cls, name: str):
            """Get encryption algorithm from string."""
            name = name.upper()
            alg_dict = cls.get_dict()
            if name not in alg_dict:
                raise ValueError(f"Encryption permission '{name}' is invalid. Valid options are: {', '.join(alg_dict.keys())}.")
            return alg_dict[name]

    class EncryptionAlgorithm(Enum):
        """PDF encryption algorithms."""

        AES_256 = "AES-256"
        AES_256_R5 = "AES-256-R5"
        AES_128 = "AES-128"
        RC4_128 = "RC4-128"
        RC4_40 = "RC4-40"

        @classmethod
        def get_dict(cls):
            """Get dictionary of encryption algorithms."""
            return {
                "AES-256": PyPDFBackend.EncryptionAlgorithm.AES_256,
                "AES-256-R5": PyPDFBackend.EncryptionAlgorithm.AES_256_R5,
                "AES-128": PyPDFBackend.EncryptionAlgorithm.AES_128,
                "RC4-128": PyPDFBackend.EncryptionAlgorithm.RC4_128,
                "RC4-40": PyPDFBackend.EncryptionAlgorithm.RC4_40,
            }

        @classmethod
        def from_str(cls, name: str):
            """Get encryption algorithm from string."""
            alg_dict = cls.get_dict()
            if name not in alg_dict:
                raise ValueError(f"Encryption algorithm '{name}' is invalid. Valid options are: {', '.join(alg_dict.keys())}.")
            return alg_dict[name]

    @staticmethod
    def len(
            input_file: str | Path,
    ) -> int:
        """
        Get number of pages of input file.

        :param input_file: Input PDF file.

        :return: Number of pages.

        :raises FileNotFoundError: if input file not found
        """
        PyPDFBackend.check_file_exists(input_file)

        with PdfReader(input_file) as reader:
            return len(reader.pages)

    def __init__(
        self,
        verbose: bool = False,
    ):
        """
        Initialize the ``pypdf`` backend

        :param verbose: Verbose logging. Defaults to False.      
        """
        super().__init__()
        self._verbose = verbose

    def merge(self,
              output_file: str | Path,
              input_files: Sequence[str | Path],
              password: str | None = None,
              progress_callback: Callable[[float], Any] | None = None,
              ):
        """
        Merge input files into an output.

        :param output_file: Output PDF file.
        :param input_files: Input PDF files. 
        :param password: Decryption password. Defaults to None.
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found.
        """
        output_file = Path(output_file).with_suffix(".pdf")
        completed = 0.0
        file_progress = 100.0 / len(input_files)

        with PdfWriter() as writer:
            for idx, input_file in enumerate(input_files, start=1):
                input_file = Path(input_file)
                self.check_file_exists(input_file)

                with PdfReader(input_file) as reader:
                    if password and reader.is_encrypted:
                        reader.decrypt(password)

                    pages_len = len(reader.pages)
                    for page in reader.pages:
                        writer.add_page(page)

                        completed += file_progress / pages_len
                        if progress_callback:
                            progress_callback(completed)
            if progress_callback:
                progress_callback(100.0)
            writer.write(output_file)

    def split(self,
              output_file: str | Path,
              input_file: str | Path,
              password: str | None = None,
              progress_callback: Callable[[float], Any] | None = None,
              ):
        """
        Split input files into 1-page PDF output files.

        :param output_file: Output PDF file (will be appended with ``_X.pdf`` where ``X`` is the page number)
        :param input_file: Input PDF file
        :param password: Decryption password. Defaults to None.
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix("")

        with PdfReader(input_file) as reader:
            if password and reader.is_encrypted:
                reader.decrypt(password)

            pages_len = len(reader.pages)
            for i, page in enumerate(reader.pages):
                with PdfWriter() as writer:
                    writer.add_page(page)
                    writer.write(f"{output_path}_{i + 1}.pdf")

                    progress = 100.0 * (float(i) / pages_len)
                    if progress_callback:
                        progress_callback(progress)

    def extract(self,
                output_file: str | Path,
                input_file: str | Path,
                pages: Sequence[int],
                password: str | None = None,
                progress_callback: Callable[[float], Any] | None = None,
                ):
        """
        Extract specific pages from input files into a PDF output file.

        :param output_file: Output PDF file
        :param input_file: Input PDF file
        :param pages: List of pages to extract (0-indexed).
        :param password: Password used to decrypt file, if needed. Defaults to None (do not decrypt).
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix(".pdf")

        with PdfReader(input_file) as reader, PdfWriter() as writer:
            if password and reader.is_encrypted:
                reader.decrypt(password)

            pdf_len = len(reader.pages)
            pages_len = len(pages)
            for idx, page_num in enumerate(pages, start=1):
                if pdf_len <= page_num:
                    raise ValueError(f"PDF '{input_file}' has only {len(reader.pages)} pages. Cannot extract page {page_num + 1}.")
                writer.add_page(reader.pages[page_num])

                progress = 100.0 * (float(idx) / pages_len)
                if progress_callback:
                    progress_callback(progress)
            if progress_callback:
                progress_callback(100.0)
            writer.write(output_path)

    def rotate(self,
               output_file: str | Path,
               input_file: str | Path,
               rotations: dict[int, int],
               decrypt_password: str | None = None,
               progress_callback: Callable[[float], Any] | None = None,
               ):
        """
        Rotate specific pages from input files, generating a PDF output file.

        :param output_file: Output PDF file.
        :param input_file: Input PDF file.
        :param rotations: Dict format { page_num: rotation_degrees }. Degree must be 0 or multiples of 90 degrees - positive or negative.
        :param decrypt_password: Password used to decrypt file, if needed. Defaults to None (do not decrypt).
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        :raises ValueError: if rotation degree is invalid (valid values are 0 or multiples of 90 degrees - positive or negative).        
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix(".pdf")

        with PdfReader(input_file) as reader, PdfWriter() as writer:
            if decrypt_password and reader.is_encrypted:
                reader.decrypt(decrypt_password)

            pages_len = len(reader.pages)
            for i, page in enumerate(reader.pages):
                rotation = int(rotations.get(i, 0))

                # parse rotation argument
                rotation = normalize_degree(rotation)

                if rotation not in (0, 90, 180, 270):
                    raise ValueError(f"Page '{i}' rotation {rotation} is invalid. Rotation must be 0, 90, 180 or 270 degrees.")

                # execute page rotation
                logger.debug(f"Rotating page {i} by {rotation} deg")
                if rotation > 0:
                    page.rotate(rotation)  # clockwise: 90, 180, 270
                writer.add_page(page)

                progress = 100.0 * (float(i) / pages_len)
                if progress_callback:
                    progress_callback(progress)

            if progress_callback:
                progress_callback(100.0)
            # save output file
            writer.write(output_path)

    def encrypt(self,
                output_file: str | Path,
                input_file: str | Path,
                owner_password: str,
                user_password: str | None = None,
                decrypt_password: str | None = None,
                permissions: Iterable[EncryptionPermission] | None = None,
                encryption_algorithm: EncryptionAlgorithm = EncryptionAlgorithm.AES_256,
                progress_callback: Callable[[float], Any] | None = None,
                ):
        """
        Encrypt input file, generating a protected PDF output file.

        **Encryption algorithms:**
        - ``AES-256``    (high security, compatible with 2008+ PDF readers - **RECOMMENDED**)
        - ``AES-128``    (good security, good compatibility - **RECOMMENDED**)
        - ``AES-256-R5`` (high security,  lowest compatibility - not recommended)
        - ``RC4-128``    (weak security, highest compatibility - not recommended)
        - ``RC4-40``     (weakest security - not recommended)

        :param output_file: Output PDF file.
        :param input_file: Input PDF file.

        :param owner_password: Owner password for encryption. Owner has ALL PERMISSIONS in the output PDF file.
        :param user_password:  User password for encryption. User has ONLY THE PERMISSIONS specified in the arguments. Defaults to None (user and owner password are the same).
        :param decrypt_password: Password used to decrypt file, if needed. Defaults to None (do not decrypt).

        :param permissions: List of permissions to grant to the user. If None, no permissions are granted (user can only open the file). Format: [``EncryptionPermission.ANNOTATE``, ``EncryptionPermission.PRINT_LOW_QUALITY``, ...]. Defaults to None.

        :param encryption_algorithm: Encryption algorithm used. Defaults to "AES-256" (for enhanced security and compatibility).
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix(".pdf")

        with PdfReader(input_file) as reader, PdfWriter() as writer:
            if decrypt_password and reader.is_encrypted:
                reader.decrypt(decrypt_password)

            # set user permissions
            permissions_flag = PyPDFBackend.EncryptionPermission.NONE.get_value()
            for perm in permissions or []:
                permissions_flag |= perm.get_value()

            # add pages into writer
            pages_len = len(reader.pages)
            for i, page in enumerate(reader.pages, start=0):
                writer.add_page(page)
                progress = 100.0 * (float(i) / pages_len)
                if progress_callback:
                    progress_callback(progress)

            # Encrypt with user and owner passwords
            writer.encrypt(
                owner_password=owner_password,
                user_password=user_password if user_password else owner_password,
                permissions_flag=permissions_flag,
                algorithm=encryption_algorithm.value,
                use_128bit=True,
            )

            # save output file
            writer.write(output_path)

            if progress_callback:
                progress_callback(100.0)

    def decrypt(self,
                output_file: str | Path,
                input_file: str | Path,
                password: str,
                progress_callback: Callable[[float], Any] | None = None,
                ):
        """
        Decrypt input file, generating a non-protected PDF output file.

        :param output_file: Output PDF file.
        :param input_file: Input PDF file.
        :param password: Password used to decrypt file.
        :param progress_callback: Progress callback (0-100). Defaults to None.

        :raises FileNotFoundError: if input file not found
        """
        self.check_file_exists(input_file)
        output_path = Path(output_file).with_suffix(".pdf")

        with PdfReader(input_file) as reader, PdfWriter() as writer:
            # decrypt file
            if reader.is_encrypted:
                reader.decrypt(password)

            # Generate decrypted pdf
            pages_len = len(reader.pages)
            for i, page in enumerate(reader.pages, start=0):
                writer.add_page(page)
                progress = 100.0 * (float(i) / pages_len)
                if progress_callback:
                    progress_callback(progress)

            # save file
            writer.write(output_path)
            if progress_callback:
                progress_callback(100.0)


__all__ = [
    "PyPDFBackend",
]



# src/file_conversor/backend/gui/_api/pdf/encrypt.py

from flask import json, render_template, request, url_for
from pathlib import Path
from typing import Any

# user-provided modules
from file_conversor.backend.gui.flask_api import FlaskApi
from file_conversor.backend.gui.flask_api_status import FlaskApiStatus

from file_conversor.cli.pdf.encrypt_cmd import execute_pdf_encrypt_cmd, PyPDFBackend

from file_conversor.config import Configuration, Environment, Log, State
from file_conversor.config.locale import get_translation

# Get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger()

EncryptionPermission = PyPDFBackend.EncryptionPermission


def _api_thread(params: dict[str, Any], status: FlaskApiStatus) -> None:
    """Thread to handle PDF encryption."""
    logger.debug(f"PDF encryption thread received: {params}")
    input_files = [Path(i) for i in params['input-files']]
    output_dir = Path(params['output-dir'])

    encrypt_algo = str(params['pdf-encryption-algorithm'])
    owner_password = str(params['owner-password'])
    user_password = params['user-password'] or None

    permissions: list[EncryptionPermission] = [
        EncryptionPermission.NONE if not bool(params['annotate']) else EncryptionPermission.ANNOTATE,
        EncryptionPermission.NONE if not bool(params['fill_forms']) else EncryptionPermission.FILL_FORMS,
        EncryptionPermission.NONE if not bool(params['modify']) else EncryptionPermission.MODIFY,
        EncryptionPermission.NONE if not bool(params['modify_pages']) else EncryptionPermission.MODIFY_PAGES,
        EncryptionPermission.NONE if not bool(params['copy']) else EncryptionPermission.COPY,
        EncryptionPermission.NONE if not bool(params['accessibility']) else EncryptionPermission.ACCESSIBILITY,
        EncryptionPermission.NONE if not bool(params['print_lq']) else EncryptionPermission.PRINT_LQ,
        EncryptionPermission.NONE if not bool(params['print_hq']) else EncryptionPermission.PRINT_HQ,
    ]

    execute_pdf_encrypt_cmd(
        input_files=input_files,
        decrypt_password=None,
        owner_password=owner_password,
        user_password=user_password,
        permissions=permissions,
        encrypt_algo=encrypt_algo,
        output_dir=output_dir,
        progress_callback=status.set_progress,
    )


def api_pdf_encrypt():
    """API endpoint to encrypt PDF documents."""
    logger.info(f"[bold]{_('PDF encryption requested via API.')}[/]")
    return FlaskApi.execute_response(_api_thread)




# src\file_conversor\cli\video\compress_cmd.py
import typer

from pathlib import Path
from typing import Annotated, List

from rich import print

# user-provided modules
from file_conversor.backend import FFmpegBackend

from file_conversor.cli.video._ffmpeg_cmd_helper import FFmpegCmdHelper, EXTERNAL_DEPENDENCIES

from file_conversor.cli.video._typer import TRANSFORMATION_PANEL as RICH_HELP_PANEL
from file_conversor.cli.video._typer import COMMAND_NAME, COMPRESS_NAME

from file_conversor.config import Environment, Configuration, State, Log
from file_conversor.config.locale import get_translation

from file_conversor.utils import ProgressManager, CommandManager
from file_conversor.utils.typer_utils import FormatOption, InputFilesArgument, OutputDirOption, TargetFileSizeOption, VideoEncodingSpeedOption, VideoQualityOption

from file_conversor.system.win.ctx_menu import WinContextCommand, WinContextMenu

# get app config
CONFIG = Configuration.get_instance()
STATE = State.get_instance()
LOG = Log.get_instance()

_ = get_translation()
logger = LOG.getLogger(__name__)

typer_cmd = typer.Typer()


def register_ctx_menu(ctx_menu: WinContextMenu):
    icons_folder_path = Environment.get_icons_folder()
    # IMG2PDF commands
    for ext in FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS:
        ctx_menu.add_extension(f".{ext}", [
            WinContextCommand(
                name="compress",
                description="Compress",
                command=f'cmd.exe /c "{Environment.get_executable()} "{COMMAND_NAME}" "{COMPRESS_NAME}" "%1""',
                icon=str(icons_folder_path / "compress.ico"),
            ),
        ])


# register commands in windows context menu
ctx_menu = WinContextMenu.get_instance()
ctx_menu.register_callback(register_ctx_menu)


@typer_cmd.command(
    name=COMPRESS_NAME,
    rich_help_panel=RICH_HELP_PANEL,
    help=f"""
        {_('Compress a video file to a target file size.')}

        {_('Outputs an video file with _compressed at the end.')}
    """,
    epilog=f"""
        **{_('Examples')}:**

        - `file_conversor {COMMAND_NAME} {COMPRESS_NAME} input_file.avi -od D:/Downloads --target-size 30M`

        - `file_conversor {COMMAND_NAME} {COMPRESS_NAME} input_file1.mp4 -ts 50M`
    """)
def compress(
    input_files: Annotated[List[Path], InputFilesArgument(FFmpegBackend.SUPPORTED_IN_VIDEO_FORMATS)],

    target_size: Annotated[str, TargetFileSizeOption(prompt=f"{_("Target file size (size[K|M|G]) [0 = do not limit output file size]")}")],

    video_encoding_speed: Annotated[str | None, VideoEncodingSpeedOption(FFmpegBackend.ENCODING_SPEEDS)] = CONFIG["video-encoding-speed"],
    video_quality: Annotated[str | None, VideoQualityOption(FFmpegBackend.QUALITY_PRESETS)] = CONFIG["video-quality"],

    file_format: Annotated[str, FormatOption(FFmpegBackend.SUPPORTED_OUT_VIDEO_FORMATS)] = CONFIG["video-format"],

    output_dir: Annotated[Path, OutputDirOption()] = Path(),
):
    ffmpeg_cmd_helper = FFmpegCmdHelper(
        install_deps=CONFIG['install-deps'],
        verbose=STATE["verbose"],
        overwrite_output=STATE["overwrite-output"],
    )

    # Set arguments for FFmpeg command helper
    ffmpeg_cmd_helper.set_input(input_files)
    ffmpeg_cmd_helper.set_output(file_format=file_format, out_stem="_compressed", output_dir=output_dir)

    ffmpeg_cmd_helper.set_video_settings(encoding_speed=video_encoding_speed, quality=video_quality)
    ffmpeg_cmd_helper.set_target_size(target_size)

    ffmpeg_cmd_helper.execute()


__all__ = [
    "typer_cmd",
    "EXTERNAL_DEPENDENCIES",
]
